#ifndef CLASS_FACTION_ELEMENT
#define CLASS_FACTION_ELEMENT

#include <vector>

#include "Element.h"

/* FACTIONS */
#define F_NO_FACTION 0
#define F_MERCENARIES 1
#define F_NGG 2
#define F_SWA 3


/// Element that could belong to a faction
class FactionElement : public virtual Element
{
	private:
        /// id of the faction
        int a_faction;
        /// flag that is true if the object is selected
        bool a_selected;
        /// icon image of the element
        graphic::Image * a_icon;
	
    public:
        /// Constructor that specify a faction
        FactionElement(int r, int c, std::vector < graphic::Image * > images, int faction = F_NO_FACTION)
        {
            // set faction
            a_faction = faction;
            // not selected by default
            a_selected = false;
    
            // set the icon for the element
            a_icon = images[(images.size() - 1)];
        };
        
        /// Destroyer    
        virtual ~FactionElement() {};
     
        /// return the faction id
        int GetFaction() { return a_faction; };
        /// set the faction id
        void SetFaction(int faction) { a_faction = faction; };    
        
        /// set selected flag to true
        void Select() { a_selected = true; };
        /// set the selected flag to false
        void Deselect() { a_selected = false; };
        
        /// return true if the element is selected
        bool IsSelected() { return a_selected; };
        
        /// return the icon image
        graphic::Image * GetIcon() { return a_icon; };
};

#endif
