#include "Functions.h"

using namespace graphic;
using namespace std;

// return screen coordinates from iso ones origin represents the origin of 
// the isometric plane
FloatPoint * iso2scr(Point iso, Point origin)
{
    return iso2scr(iso.x, iso.y, origin.x, origin.y);  
}
        
// return screen coordinates from iso ones origin represents the origin of 
// the isometric plane
FloatPoint * iso2scr(Sint16 ix, Sint16 iy, Sint16 ox, Sint16 oy)   
{
	FloatPoint * scr = new FloatPoint;

	//initial translation
	ix -= ox; 
	iy -= oy; 
	
	//back to 45
	iy *= 2;

	//back to orthogonal
	scr->x = ix * COS_MIN_45 - iy * SIN_MIN_45;
	scr->y = ix * SIN_MIN_45 + iy * COS_MIN_45;
	
	return scr;
}

// return the relative orientation of an origin point respect another
int RelDirection(Point * origin, Point * direction) 
{ return RelDirection(origin->x, origin->y, direction->x, direction->y); }
        
// return the relative orientation of an origin cell respect another
int RelDirection(Sint16 ox, Sint16 oy, Sint16 dx, Sint16 dy)
{
    FloatPoint * dir = new FloatPoint;       
    int d;

    dir->x = dx - ox;
    dir->y = dy - oy;
        
    if((dir->x >= (- dir->y / TAN_67_5)) && (dir->x <= (- dir->y / TAN_22_5)) && (dir->y <= 0))
        d = NORTH_EAST;
    else if((dir->x >= (dir->y / TAN_67_5)) && (dir->x <= (- dir->y / TAN_67_5)) && (dir->y <= 0))
        d = NORTH;
    else if((dir->y >= (- dir->x / TAN_67_5)) && (dir->y <= ( dir->x / TAN_67_5)) && (dir->x >= 0))
        d = EAST;
    else if((dir->y <= (dir->x * TAN_67_5)) && (dir->y >= (dir->x / TAN_67_5)) && (dir->x >= 0))
        d = SOUTH_EAST;    
    else if((dir->x <= (- dir->y / TAN_67_5)) && (dir->x >= (- dir->y / TAN_22_5)) && (dir->y > 0))
        d= SOUTH_WEST;  
    else if((dir->x <= (dir->y / TAN_67_5)) && (dir->x >= (- dir->y / TAN_67_5)) && (dir->y > 0))
        d = SOUTH;
    else if((dir->y <= (- dir->x / TAN_67_5)) && (dir->y >= ( dir->x / TAN_67_5)) && (dir->x < 0))
        d =  WEST;    
    else
        d = NORTH_WEST;    
    
    delete dir;
    
    return d;
}


// temporary menu
void menu(Options & op)
{
    // used colors
    SDL_Color sel_color = {0xED, 0x70, 0x00, 0};
    SDL_Color unsel_color = {0xAA, 0x60, 0x00, 0};
    SDL_Color black = {0,0,0,0};

    // used fonts
    string menu_font = "data/font/Distres2.ttf";
    string txt_font = "data/font/tahomabd.ttf";
    
    // background images
    Image bg_800("data/img/mars1_800.png");
    Image bg_1024("data/img/mars1_1024.png");
    Image * bg = & bg_1024;
    
    // menu text
    Text txt1("TileSet 1", unsel_color, menu_font, DIM_FONT_MENU);
    Text txt1_s("TileSet 1", sel_color, menu_font, DIM_FONT_MENU);
    Text txt2("TileSet 2", unsel_color, menu_font, DIM_FONT_MENU);
    Text txt2_s("TileSet 2", sel_color, menu_font, DIM_FONT_MENU);
    Text txt3("TileSet 3", unsel_color, menu_font, DIM_FONT_MENU);
    Text txt3_s("TileSet 3", sel_color, menu_font, DIM_FONT_MENU);
                               
    txt1.SetPosition(X_R2_OFFSET, op.ris.h / 3 + 50);
    txt2.SetPosition(X_R2_OFFSET , op.ris.h / 3 + 100);
    txt3.SetPosition(X_R2_OFFSET , op.ris.h / 3 + 150);
    txt1_s.SetPosition(X_R2_OFFSET , op.ris.h / 3 + 50);
    txt2_s.SetPosition(X_R2_OFFSET , op.ris.h / 3 + 100);
    txt3_s.SetPosition(X_R2_OFFSET , op.ris.h / 3 + 150);
    
    AlphaImage comm("data/img/commands.png");
    comm.SetPosition(0, op.ris.h - comm.GetH());

    // blit background
    blitter->Blit(*bg);
    
    // blit menu
    blitter->Blit(txt1_s);
    blitter->Blit(txt2);
    blitter->Blit(txt3);
    
    // blit help
    blitter->Blit(comm);

    screen->Update();
    
    bool done = false;
    SDL_Event event;
    
    op.choice = 1;
    
    //game loop 
    while(!done)    
    {
        while(SDL_PollEvent(&event))
        {
            //the program quits if someone press the x simbol of the window with the mouse 
            if( event.type == SDL_QUIT )  
            {
                op.choice = 0;
                done = true;  
            }
            else if( event.type == SDL_KEYDOWN )
            {
                //quit if someone press ESC 
                switch(event.key.keysym.sym)
                {
                    case SDLK_ESCAPE:
                        op.choice = 0;
                        done = true; 
                        break;
                        
                    case SDLK_f:
                        // save screen
                        screen->Backup();
                        // go in fullscreen mode
                        screen->FullScreen();
                        //restore screen
                        screen->Restore();
                        
                        screen->Update();
                        break;
                        
                    case SDLK_w:
                        // save window 
                        screen->Backup();
                        // go in window mode
                        screen->Window();
                        // restore window
                        screen->Restore();
                        
                        screen->Update();
                        break;
                        
                    // switch menu choice
                    case SDLK_UP:
                        switch(op.choice)
                        {
                            case 1:
                                blitter->Blit(txt1);
                                blitter->Blit(txt3_s);
                                            
                                op.choice = 3;
                                break;
                            
                            case 2:
                                blitter->Blit(txt1_s);
                                blitter->Blit(txt2);
                                
                                op.choice = 1;    
                                break;
                            
                            case 3:
                                blitter->Blit(txt2_s);
                                blitter->Blit(txt3);
                            
                                op.choice = 2;    
                                break;
                                
                            default:
                                break;
                        }
                        
                        screen->Update();
                        break;         
         
                    case SDLK_DOWN:
                        switch(op.choice)
                        {
                            case 1:
                                blitter->Blit(txt1);
                                blitter->Blit(txt2_s);
                                
                                op.choice = 2;
                                break;
                            
                            case 2:
                                blitter->Blit(txt2);
                                blitter->Blit(txt3_s);
                            
                                op.choice = 3;    
                                break;
                            
                            case 3:
                                blitter->Blit(txt1_s);
                                blitter->Blit(txt3);
                            
                                op.choice = 1;    
                                break;
                                
                            default:
                                break;
                        }
                        
                        screen->Update();
                        break;
                    
                    // end
                    case SDLK_RETURN:
                        done = true;
                        break;
                    
                    // 800x600
                    case SDLK_F1:
                        screen->SetResolution(RIS1_W, RIS1_H);
                        
                        op.ris.w = RIS1_W;
                        op.ris.h = RIS1_H;
                        
                        screen->Fill(black);
                        
                        txt1.SetPosition(X_R1_OFFSET, op.ris.h / 3 + 50);
                        txt2.SetPosition(X_R1_OFFSET , op.ris.h / 3 + 100);
                        txt3.SetPosition(X_R1_OFFSET , op.ris.h / 3 + 150);
                        txt1_s.SetPosition(X_R1_OFFSET , op.ris.h / 3 + 50);
                        txt2_s.SetPosition(X_R1_OFFSET , op.ris.h / 3 + 100);
                        txt3_s.SetPosition(X_R1_OFFSET , op.ris.h / 3 + 150);
                    
                        comm.SetPosition(0, op.ris.h - comm.GetH());

                        bg = &bg_800;
                        
                        blitter->Blit(*bg);
      
                        switch(op.choice)
                        {
                            case 1:
                                blitter->Blit(txt1_s);
                                blitter->Blit(txt2);
                                blitter->Blit(txt3);

                                break;
                                
                            case 2:
                                blitter->Blit(txt1);
                                blitter->Blit(txt2_s);
                                blitter->Blit(txt3);

                                break;
                                
                            case 3:
                                blitter->Blit(txt1);
                                blitter->Blit(txt2);
                                blitter->Blit(txt3_s);

                                break;
                
                            default:
                                break;
                        }

                        blitter->Blit(comm);
                        
                        screen->Update();
        
                        break; 
                    
                    // 1024x768
                    case SDLK_F2:       
                        screen->SetResolution(RIS2_W, RIS2_H);
                        
                        op.ris.w = RIS2_W;
                        op.ris.h = RIS2_H;
                        
                        screen->Fill(black);
                        
                        txt1.SetPosition(X_R2_OFFSET, op.ris.h / 3 + 50);
                        txt2.SetPosition(X_R2_OFFSET , op.ris.h / 3 + 100);
                        txt3.SetPosition(X_R2_OFFSET , op.ris.h / 3 + 150);
                        txt1_s.SetPosition(X_R2_OFFSET , op.ris.h / 3 + 50);
                        txt2_s.SetPosition(X_R2_OFFSET , op.ris.h / 3 + 100);
                        txt3_s.SetPosition(X_R2_OFFSET , op.ris.h / 3 + 150);
      
                        comm.SetPosition(0, op.ris.h - comm.GetH());
                        
                        bg = &bg_1024;
                        
                        blitter->Blit(*bg);
      
      
                        switch(op.choice)
                        {
                            case 1:
                                blitter->Blit(txt1_s);
                                blitter->Blit(txt2);
                                blitter->Blit(txt3);
        
                                break;
                                
                            case 2:
                                blitter->Blit(txt1);
                                blitter->Blit(txt2_s);
                                blitter->Blit(txt3);
    
                                break;
                                
                            case 3:
                                blitter->Blit(txt1);
                                blitter->Blit(txt2);
                                blitter->Blit(txt3_s);
        
                                break;
                
                            default:
                                break;
                        }

                        blitter->Blit(comm);

                        screen->Update();
                        
                        break;
                        
                    default:
                        break;
                }
            }
        }
        
        SDL_Delay(1);
    }
}
