#include "IsoFow.h"

using namespace std;
using namespace graphic;

// base constructor, use a map to choose tiles
IsoFow::IsoFow(IsoTilesMap * tm, Image * image)			
{
	// store the new Image
    a_image = image;
	
	a_image->SetOpacity(FOW_DEFAULT_OPACITY);
     
    // storing IsoTilesMap pointer
    a_tm = tm;
 
    a_tile_width = a_image->GetW();    
    a_tile_height = a_image->GetH();    
    a_square_side = a_tile_height * SQRT2;

    a_num_rows = a_tm->GetNumRows();        
    a_num_cols = a_tm->GetNumCols();
        
    a_origin.x = a_tm->GetOriginX();
    a_origin.y = a_tm->GetOriginY();
}


// blit part of the FOW on a surface
void IsoFow::BlitLayer(graphic::BaseSurface & dst_surf, SDL_Rect * limits_rect)
{
	Sint16 l_limit = limits_rect->x ;
	Sint16 r_limit = limits_rect->x + limits_rect->w;
	Sint16 u_limit = limits_rect->y;
	Sint16 b_limit = limits_rect->y + limits_rect->h;	
	
	// temporary Point
	Point * p = new Point;
   
    FloatPoint * start, * end;
    int s_row, s_col, e_row, e_col;
    
    // compute start row =>  top, right corner
     start = iso2scr(r_limit, u_limit, a_origin.x, a_origin.y);            
    s_row = (int)(start->y / a_square_side) - 1;           
    if(s_row < 0)
        s_row = 0;
    delete start;
   
    // compute start col => top, left corner
    start = iso2scr(l_limit, u_limit, a_origin.x, a_origin.y);    
    s_col = (int)(start->x / a_square_side) - 1;
    if(s_col < 0)
        s_col = 0;
    delete start;    
    
    // compute end row => bottom, left corner
    end = iso2scr(l_limit, b_limit, a_origin.x, a_origin.y);    
    // row and col of the end Point
    e_row = (int)(end->y / a_square_side) + 1;
    if(e_row > a_num_rows)
        e_row = a_num_rows;
    delete end;
    
    // compute end col => bottom, right corner
    end = iso2scr(r_limit, b_limit, a_origin.x, a_origin.y);    
    e_col = (int)(end->x / a_square_side) + 1;                
    if(e_col > a_num_cols)
        e_col = a_num_cols;
    delete end;   

	for(int r = s_row; r < e_row; r++)
	{
		for(int c = s_col; c < e_col; c++)
		{
			p->x = a_tm->GetObjectX(r,c);
			p->y = a_tm->GetObjectY(r,c);
			
			if( (a_tm->IsVisible(r, c) == PARTIALLY_VISIBLE) &&
				(p->x + a_tile_width >= l_limit) && (p->x <= r_limit) && 
				(p->y + a_tile_height >= u_limit) && (p->y <= b_limit) )
			{
				//set blit position of the image
                a_image->SetPosition(p->x - l_limit, p->y - u_limit);
				//blit 
				blitter->Blit(*(a_image), dst_surf);
			}
		}
	}   
	
	delete p;
}

// blit all the FOW on a surface
void IsoFow::BlitLayer(graphic::BaseSurface & dst_surf)
{
	int num_rows = a_tm->GetNumRows();
	int num_cols = a_tm->GetNumCols();
	
	// temporary Point
	Point * p = new Point;
	
	for(int r = 0; r < num_rows; r++)
	{
		for(int c = 0; c < num_cols; c++)
		{
			p->x = a_tm->GetObjectX(r,c);
			p->y = a_tm->GetObjectY(r,c);
			
			if(!a_tm->IsVisible(r, c))
			{
				//set blit position of the image
				a_image->SetPosition(p->x, p->y);
				//blit 
				blitter->Blit(*(a_image), dst_surf);
			}
		}
	}
	
	delete p;
}
