#include "IsoGrid.h"

using namespace std;
using namespace graphic;

// base constructor, use a map to choose tiles
IsoGrid::IsoGrid(IsoTilesMap * tm, Image * v_image, Image * pv_image, Image * nv_image) 			
{
	// store Image
    a_v_cell = v_image;
    a_pv_cell = pv_image;
    a_nv_cell = nv_image;
	
	a_v_cell->SetOpacity(GRID_V_DEFAULT_OPACITY);
	a_pv_cell->SetOpacity(GRID_PV_DEFAULT_OPACITY);
	a_nv_cell->SetOpacity(GRID_NV_DEFAULT_OPACITY);

	
	// storing IsoTilesMap pointer
	a_tm = tm;
   
    a_tile_width = a_v_cell->GetW();    
    a_tile_height = a_v_cell->GetH();    
    a_square_side = a_tile_height * SQRT2;

    a_num_rows = a_tm->GetNumRows();        
    a_num_cols = a_tm->GetNumCols();
        
    a_origin.x = a_tm->GetOriginX();
    a_origin.y = a_tm->GetOriginY();
}


// blit the grid on a surface
void IsoGrid::BlitLayer(graphic::BaseSurface & dst_surf, SDL_Rect * limits_rect)
{
	Sint16 l_limit = limits_rect->x ;
	Sint16 r_limit = limits_rect->x + limits_rect->w;
	Sint16 u_limit = limits_rect->y;
	Sint16 b_limit = limits_rect->y + limits_rect->h;	
	
	// temporary Point
	Point * p = new Point; 
 
    FloatPoint * start, * end;
    int s_row, s_col, e_row, e_col;
    
    // compute start row =>  top, right corner
    start = iso2scr(r_limit, u_limit, a_origin.x, a_origin.y);            
    s_row = (int)(start->y / a_square_side) - 1;           
    if(s_row < 0)
        s_row = 0;
    delete start;
   
    // compute start col => top, left corner
    start = iso2scr(l_limit, u_limit, a_origin.x, a_origin.y);    
    s_col = (int)(start->x / a_square_side) - 1;
    if(s_col < 0)
        s_col = 0;
    delete start;    
    
    // compute end row => bottom, left corner
    end = iso2scr(l_limit, b_limit, a_origin.x, a_origin.y);    
    // row and col of the end Point
    e_row = (int)(end->y / a_square_side) + 1;
    if(e_row > a_num_rows)
        e_row = a_num_rows;
    delete end;
    
    // compute end col => bottom, right corner
    end = iso2scr(r_limit, b_limit, a_origin.x, a_origin.y);    
    e_col = (int)(end->x / a_square_side) + 1;                
    if(e_col > a_num_cols)
         e_col = a_num_cols;
    delete end;   

	for(int r = s_row; r < e_row; r++)
	{
		for(int c = s_col; c < e_col; c++)
		{
			p->x = a_tm->GetObjectX(r,c);
			p->y = a_tm->GetObjectY(r,c);
			
			if( (p->x + a_tile_width >= l_limit) && (p->x <= r_limit) && 
		   	   (p->y + a_tile_height >= u_limit) && (p->y <= b_limit) )
			{
				// obtain tile position
				p->x -= l_limit;
				p->y -= u_limit;

				//blit 
				if(a_tm->IsVisible(r,c) == NOT_VISIBLE)
				{
					//set blit position of the image
					a_nv_cell->SetPosition(p->x, p->y);	
					blitter->Blit(*(a_nv_cell), dst_surf);
				}
				else if(a_tm->IsVisible(r,c) == VISIBLE)
				{
					//set blit position of the image
					a_v_cell->SetPosition(p->x, p->y);	
					blitter->Blit(*(a_v_cell), dst_surf);
				}
				else if(a_tm->IsVisible(r,c) == PARTIALLY_VISIBLE)
				{
					//set blit position of the image
					a_pv_cell->SetPosition(p->x, p->y);	
					blitter->Blit(*(a_pv_cell), dst_surf);
				}
			}
		}
	}
	
	delete p;
}

// blit all the grid on a surface
void IsoGrid::BlitLayer(graphic::BaseSurface & dst_surf)
{
	int num_rows = a_tm->GetNumRows();
	int num_cols = a_tm->GetNumCols();
	
	for(int r = 0; r < num_rows; r++)
	{
		for(int c = 0; c < num_cols; c++)
		{
			//blit 
			if(a_tm->IsVisible(r,c))
			{
				//set blit position of the image
				a_v_cell->SetPosition(a_tm->GetObjectX(r,c), a_tm->GetObjectY(r,c));	
				blitter->Blit(*(a_v_cell), dst_surf);
			}
			else
			{
				//set blit position of the image
				a_nv_cell->SetPosition(a_tm->GetObjectX(r,c), a_tm->GetObjectY(r,c));	
				blitter->Blit(*(a_nv_cell), dst_surf);
			}
		}
	}
}
