#ifndef CLASS_ISO_GRID
#define CLASS_ISO_GRID

#include "graphic/Blitter.h"
#include "graphic/Image.h"
#include "IsoTilesMap.h"
#include "Layer.h"
#include "Tile.h"

#define GRID_V_DEFAULT_OPACITY 30
#define GRID_PV_DEFAULT_OPACITY 20
#define GRID_NV_DEFAULT_OPACITY 50

/// A grid of tiles
class IsoGrid : public Layer
{
	private:
		/// image associated to a visible tile
		graphic::Image * a_v_cell;
        /// image associated to a partially visible tile
        graphic::Image * a_pv_cell;
		/// image associated to a NOT visible tile
		graphic::Image * a_nv_cell;

		/// pointer to the Isometric Tiles Map
		IsoTilesMap * a_tm;
  
        /// width of the tile
        Uint16 a_tile_width;
        /// height of the tile
        Uint16 a_tile_height;
        /// number of rows 
        int a_num_rows;
        /// number of columns 
        int a_num_cols;
        /// side of an orthogonal tile 
        float a_square_side;
  
        /// origin of the map
        Point a_origin;
		
	public:
		/// base constructor, use a map to choose tiles
     IsoGrid(IsoTilesMap * tm, graphic::Image * v_image, 
             graphic::Image * pv_image, graphic::Image * nv_image);
		/// destroyer
		~IsoGrid() { };
			
		/// blit the grid on a surface
		void BlitLayer(graphic::BaseSurface & dst_surf, SDL_Rect * limits_rect);
		/// blit all the grid on a surface
		void BlitLayer(graphic::BaseSurface & dst_surf);

		/// Set opacity of the grid on visible tiles
		void SetVOpacity(Uint8 opacity)	{ a_v_cell->SetOpacity(opacity); };
        /// Set opacity of the grid on not visible tiles
        void SetPVOpacity(Uint8 opacity) { a_pv_cell->SetOpacity(opacity); };
		/// Set opacity of the grid on not visible tiles
		void SetNVOpacity(Uint8 opacity) { a_nv_cell->SetOpacity(opacity); };

};

#endif
