#ifndef CLASS_ISO_MINI_MAP
#define CLASS_ISO_MINI_MAP

#include "graphic/Blitter.h"
#include "graphic/Image.h"
#include "DataTypes.h"
#include "Element.h"
#include "FactionElement.h"
#include "IsoTilesMap.h"
#include "IsoObjectsMap.h"
#include "Map.h"
#include "MiniTile.h"
#include "TileSet.h"

#define IMG_NOT_VISIBLE 0
#define IMG_PARTIALLY_VISIBLE 1
#define IMG_VISIBLE 2
#define IMG_ELEMENT 3
#define IMG_NO_FACTION 4
#define IMG_MERCENARIES 5
#define IMG_SWA 6
#define IMG_NGG 7

/// MiniMap of the scene
class IsoMiniMap : public Map
{
	private:
		/// vector containing all the used images
		TileSet * a_tile_set;
		/// number of images used by the map
		int a_num_images;
		/// width of a tile
		Uint16 a_tile_width;
		/// height of a tile
		Uint16 a_tile_height;
		/// pointer to the Isometric Tiles Map
		IsoTilesMap * a_tm;
		/// pointer to the Isometric Objects Map
		IsoObjectsMap * a_om;
		/// scale of the mini map (iso tiles width / minimap tiles width)
		int a_scale;
        /// side of an orthogonal tile 
        float a_square_side;
		
	public:
		/// base constructor, use a map to choose tiles
		IsoMiniMap(IsoTilesMap * tm, IsoObjectsMap * om, TileSet * ts);
		
		/// destructor 
		~IsoMiniMap() {};
			
		/// blit part of the mini map on a surface
		void BlitLayer(graphic::BaseSurface & dst_surf, SDL_Rect * limits_rect);
		/// blit all the mini map on a surface
		void BlitLayer(graphic::BaseSurface & dst_surf);
		
		/// update minimap tiles
		void UpdateMap();
		
		/// return the scale value
		int GetScale() { return a_scale; };
  
        /// return the cell indexes of the selected point if the selection is inside the 
        /// map, NULL otherwise
        CellInd * GetCell(Sint16 x, Sint16 y) { return NULL; }; 
};

#endif

