#include "IsoMouse.h"
#include <iostream>

using namespace std;

// Constructor
IsoMouse::IsoMouse(IsoTilesMap * tm, vector< graphic::MovableImage * > pointers) : Mouse(pointers)
{
    // indexes of the current cell pointed by the mouse
    a_cur_cell = new CellInd;
    a_cur_cell->row = -1;
    a_cur_cell->col = -1;
    a_prev_cell = new CellInd;
    a_prev_cell->row = -1;
    a_prev_cell->col = -1;
    
    // flag that is true when the current cell change
    a_changed_cell = false;
    
    // information about tiles map 
    a_square_side = tm->GetTileH() * SQRT2;
    a_num_rows = tm->GetNumRows();
    a_num_cols = tm->GetNumCols();
    a_origin = new Point;
    a_origin->x = tm->GetOriginX(); 
    a_origin->y = tm->GetOriginY(); 
    
    // orthogonal position of the mouse cursor
    a_ortho_pos = new FloatPoint;
}

// Destroyer
IsoMouse::~IsoMouse()
{
    delete a_cur_cell;
    delete a_prev_cell;
    delete a_ortho_pos;
}

void IsoMouse::SetIndexes()
{              
    FloatPoint * scr;

    scr = iso2scr(*a_abs_position, *a_origin);
    
    // store ortho position
    a_ortho_pos->x = scr->x;
    a_ortho_pos->y = scr->y;
    
    // store previous cell
    a_prev_cell->row = a_cur_cell->row;
    a_prev_cell->col = a_cur_cell->col;
            
    // row and col of the Point
    a_cur_cell->row = (int)(scr->y / a_square_side);
    a_cur_cell->col = (int)(scr->x / a_square_side);
    
    // negative adjustement
    if(scr->y < 0)
        a_cur_cell->row--;
    if(scr->x < 0)
        a_cur_cell->col--;   
    
    // check if the cell is changed
    if(a_prev_cell->row != a_cur_cell->row || a_prev_cell->col != a_cur_cell->col)
        a_changed_cell = true;
    else
        a_changed_cell = false;
    
    delete scr;    
}

// return the current cell indexes
CellInd * IsoMouse::GetCell() 
{ 
    CellInd * cell = new CellInd;
    
    cell->row = a_cur_cell->row;
    cell->col = a_cur_cell->col;       
         
    return cell;
}

// return the previous cell indexes
CellInd * IsoMouse::GetPrevCell() 
{ 
    CellInd * cell = new CellInd;
    
    cell->row = a_prev_cell->row;    
    cell->col = a_prev_cell->col;       
         
    return cell;
}

// return true if the mouse is inside the isometric map
bool IsoMouse::IsInsideMap()
{
    if(a_cur_cell->row >= 0 && a_cur_cell->row < a_num_rows &&
       a_cur_cell->col >= 0 && a_cur_cell->col < a_num_cols )
        return true;
    else
        return false;
}
