#ifndef CLASS_ISO_MOUSE
#define CLASS_ISO_MOUSE

#include <vector>

#include "graphic/MovableImage.h"
#include "DataTypes.h"
#include "Functions.h"
#include "IsoTilesMap.h"
#include "Mouse.h"

/// extends Mouse class adding methods and atributes related to iso management
class IsoMouse : public Mouse
{
	private:	
     /// indexes of the current cell pointed by the mouse
     CellInd * a_cur_cell;
     /// indexes of the previous cell pointed by the mouse
     CellInd * a_prev_cell;
     /// flag that is true when the current cell change
     bool a_changed_cell;
          
     /// a side of an isometric square transformed in an orthogonal one
     float a_square_side;
     /// number of rows of the tiles map
     int a_num_rows;
     /// number of cols of the tiles map
     int a_num_cols;
     /// origin of the tiles map
     Point * a_origin;
     
     /// orthogoanl position obtained by the screen (isometric) one
     FloatPoint * a_ortho_pos;

     /// set current indexes of the cell under the mouse
     void SetIndexes();

	public:
     /// Constructor
     IsoMouse(IsoTilesMap * tm, std::vector< graphic::MovableImage * > pointers);
     /// Destroyer
     ~IsoMouse();
     
     /// Set the current view values
     void SetView(Sint16 view_x, Sint16 view_y)
     { 
         Mouse::SetView(view_x, view_y);
         SetIndexes();
     };
     /// Set the current view values passing a point
     void SetView(Point view) { SetView(view.x, view.y); };
     
     /// Set the position passing just the screen position   
     void SetPosition(Sint16 x, Sint16 y)
     {       
         Mouse::SetPosition(x, y);   
         SetIndexes();
     };
     /// Set the position passing just the point of screen position    
     void SetPosition(Point pos) { SetPosition(pos.x, pos.y); };     
     /// Set the position passing screen position and view position   
     void SetPosition(Sint16 x, Sint16 y, Sint16 view_x, Sint16 view_y) 
     { 
         SetView(view_x, view_y);
         SetPosition(x, y);
     };
     /// Set the position passing screen position as point and view position as point
     void SetPosition(Point pos, Point view) { SetPosition(pos.x, pos.y, view.x, view.y); };
     
     /// return true if the current change is just changed
     bool IsCellChanged() { return a_changed_cell; };
     
     /// return true if the mouse is inside the isometric map
     bool IsInsideMap();
     
     /// return the current row
     int GetRow() { return a_cur_cell->row; };
     /// return the current col
     int GetCol() { return a_cur_cell->col; };
     /// return the current cell indexes
     CellInd * GetCell();
     
     /// return the previous row
     int GetPrevRow() { return a_prev_cell->row; };
     /// return the previous col
     int GetPrevCol() { return a_prev_cell->col; };
     /// return the previous cell indexes
     CellInd * GetPrevCell();
     
     /// return the absolute X iso2scr position
     float GetAbsOrthoX() { return a_ortho_pos->x; };
     /// return the absolute Y iso2scr position
     float GetAbsOrthoY() { return a_ortho_pos->y; };
};

#endif
