#ifndef CLASS_ISO_OBJECTS_MAP
#define CLASS_ISO_OBJECTS_MAP

#include <vector>

#include "Map.h"
#include "Layer.h"
#include "IsoObject.h"
#include "IsoTilesMap.h"

// coefficient used to compute the partially visible radius
#define PV_COEFFICIENT 3
#define DIAGONAL_BONUS 1

/// Example of a Doxygen compatible comment for a class
class IsoObjectsMap : public Layer
{
    private:
        /// pointer to the Tiles Map
        IsoTilesMap * a_tm;
        /// map of pointer to the objects
        std::vector < IsoObject * > a_objects_map;
        /// vector containing Iso objects
        std::vector < IsoObject * > a_objects;
        /// objects present in the current view
        std::vector < IsoObject * > a_cur_objects;
        /// number of objects contained
        int a_num_objects;
        /// number of rows of the map
        int a_num_rows;
        /// number of cols of the map
        int a_num_cols;
        
                
        /// sort objects so to blit front objects after rear objects
        void SortObjects();
        
	public:
		/// constructor
        IsoObjectsMap(IsoTilesMap * tm);
		/// destroyer
		~IsoObjectsMap() {};
		
		/// blit part of the objects map on a surface
		void BlitLayer(graphic::BaseSurface & dst_surf, SDL_Rect * limits_rect);
		/// blit the entire objects map on a surface
		void BlitLayer(graphic::BaseSurface & dst_surf);
  
        /// add an object to the Map at the [row, col] position
        void AddObject(IsoObject * obj, int row, int col);
        
        /// select the object at the x,y position, return the pointer if an object 
        /// has been selected, NULL otherwise
        IsoObject * SelectObject(Sint16 x, Sint16 y);
        /// return a pointer to the object at [row. col] cell
        IsoObject * GetObject(int row, int col) { return a_objects_map[(row * a_num_cols) + col];};

        
        /// move an object at the [row, col] position
        void MoveObject(IsoObject * obj, int row, int col);
        /// move an object at the dest_cell position
        void MoveObject(IsoObject * obj, CellInd * dest_cell) 
            { MoveObject(obj, dest_cell->row, dest_cell->col);};
        
        /// set the visibility range of an object in accord with the view direction and its
        /// visibility radius.         
        void ComputeObjVisibility(IsoObject * obj, int radius, int direction = ANY);
                
        /// return the absolute X position of the image of the object on the map
        Sint16 GetObjX(IsoObject * obj);
        
        /// return the absolute Y position of the image of the object on the map
        Sint16 GetObjY(IsoObject * obj);
};

#endif
