#ifndef CLASS_LAYER
#define CLASS_LAYER

#include "graphic/BaseSurface.h"

/// Layer of a Scene
class Layer
{
	private:
		/// level of the layer, 0 -> bottom, x>0 -> top
		int a_level;
        /// flag that is true if the flag need to be blitted
        bool a_need_blit;
        /// flag that is true if the layer is buffered
        bool a_buffered;
        
		
	protected:
		/// width of the layer
		Uint16 a_width;
		/// height of the layer
		Uint16 a_height;
		
	public:
		/// constructor
		Layer() 
        {
            a_need_blit = true;
            a_buffered = false;
        };
		/// destroyer
		virtual ~Layer() {};
		
		/// blit part of the layer on a surface
		virtual void BlitLayer(graphic::BaseSurface & dst_surf, SDL_Rect * limits_rect) = 0;
		/// blit all the layer on a surface
		virtual void BlitLayer(graphic::BaseSurface & dst_surf) = 0;

		/// get level number
		int GetLevel() { return a_level; };
		/// set level number
		void SetLevel(int lev) { a_level = lev; };
		/// increment level number

		void UpLevel() { a_level++; };
		/// decrement level number
		void DownLevel() 
		{ 
			if(a_level) 
				a_level--; 
		};
		
		/// obtain width of the layer
		Uint16 GetW() { return a_width; };
		/// obtain height of the layer
		Uint16 GetH() { return a_height; };
  
        /// set the need of blit flag
        void SetBlit(bool b = true) { a_need_blit = b; };
        /// return true if the layer need to be blitted
        bool NeedBlit() { return a_need_blit; };
        
        /// set the buffering flag
        void SetBuffering(bool b = true) { a_buffered = b; };
        /// return true if the layer need to be buffered
        bool IsBuffered() { return a_buffered; };  
};

#endif
