#ifndef CLASS_TILES_MAP
#define CLASS_TILES_MAP

#include <vector>

#include "Layer.h"
#include "Object.h"

/// A map containing objects
class Map: public Layer 
{
	protected:
		/// vector containing objects
		std::vector < Object * > a_objects;
		/// number of rows 
		int a_num_rows;
		/// number of columns 
		int a_num_cols;
		/// total number of cells of the map (row x col), useful to avoid 
		/// further multiplication
		int a_num_cells;
		/// number of objects of the map (a_objects.size())
		int a_num_objects;
		/// origin of the map
		Point a_origin;
        /// origin of the [0,0] image, top-left corner
        Point a_graphic_origin;

	public:
		/// constructor
		Map(int num_rows, int num_cols);
		/// destroyer
		virtual ~Map();
			
		/// blit part of the map on a surface
		virtual void BlitLayer(graphic::BaseSurface & dst_surf, SDL_Rect * limits_rect) = 0;
		/// blit all the layer on a surface
		virtual void BlitLayer(graphic::BaseSurface & dst_surf) = 0;

		/// Get X coordinate of the origin Point
		Sint16 GetOriginX() { return a_origin.x; };
		/// Get Y coordinate of the origin Point
		Sint16 GetOriginY() { return a_origin.y; };
  
        /// Get X coordinate of the graphic origin Point (top-left vertex of the [0, 0] tile)
        Sint16 GetGraphicOriginX() { return a_graphic_origin.x; };
        /// Get Y coordinate of the graphic origin Point (top-left vertex of the [0, 0] tile)
        Sint16 GetGraphicOriginY() { return a_graphic_origin.y; };
		
		/// Set X coordinate of the origin Point, then update all objects X value
		void SetOriginX(Sint16 new_x);
		/// Set Y coordinate of the origin Point, then update all objects Y value
		void SetOriginY(Sint16 new_y);
		
		/// Get X coordinate of the object at [row,col]
		Sint16 GetObjectX(int r, int c) { return a_objects[(r * a_num_cols) + c]->GetX(); };
		/// Get Y coordinate of the object at [row,col]
		Sint16 GetObjectY(int r, int c) { return a_objects[(r * a_num_cols) + c]->GetY(); };
		
		/// Get number of rows of the map
		int GetNumRows() { return a_num_rows; };
		/// Get number of columns of the map
		int GetNumCols() { return a_num_cols; };
		/// Get number of cells of the map
		int GetNumCells() { return a_num_cells; };
		/// Get the number of objects
		int GetNumObjects() { return a_num_objects; };	
		
		/// Get width of the map
		Uint16 GetW() { return a_width; };	
		/// Get height of map
		Uint16 GetH() { return a_height; };	
  
        /// return the cell indexes of the selected point if the selection is inside the 
        /// map, NULL otherwise
        virtual CellInd * GetCell(Sint16 x, Sint16 y) = 0;
};

#endif
