#include "MiniMapPanel.h"
//#include <iostream>

using namespace graphic;
using namespace std;

// constructor
MiniMapPanel::MiniMapPanel(IsoMiniMap * mm, Image * view_rect, SDL_Rect mm_limits)
{
    // store the minimap pointer
    a_mm = mm;
    // minimap scale
    a_mm_scale = mm->GetScale();
    
    // store the view rect image pointer
    a_view_rect = view_rect;
    
    // create the panel surface
    a_panel = new Surface(mm_limits.w, mm_limits.h);
    // create the buffer surface
    a_buffer = new Surface(mm_limits.w, mm_limits.h); 

    // background color
    a_back_color.r = 0;
    a_back_color.g = 0;
    a_back_color.b = 0;
    
    // store minimap limits
    a_mm_limits.x = mm_limits.x;
    a_mm_limits.y = mm_limits.y;
    a_mm_limits.w = mm_limits.w;
    a_mm_limits.h = mm_limits.h;   
      
    a_buffer->Fill(a_back_color);
    a_mm->BlitLayer(*a_buffer, &a_mm_limits);
    blitter->Copy(*a_buffer, *a_panel);
    blitter->Blit(*a_view_rect, *a_panel);
}

// mouse has been clicked on the panel at (x,y) point
void MiniMapPanel::MouseClicked(Sint16 x, Sint16 y)
{      
    Sint16 vx, vy;
    
    vx = x - (a_view_rect->GetW() / 2) + a_mm_limits.x;
    vy = y - (a_view_rect->GetH() / 2) + a_mm_limits.y;
    
    // update minimap rect limits
    a_mm_limits.x += x - (a_mm_limits.w / 2);                                                
    a_mm_limits.y += y - (a_mm_limits.h / 2);

    // check for limits exceeded
    if(a_mm_limits.y < (a_u_rlimit / a_mm_scale))
        a_mm_limits.y = (a_u_rlimit / a_mm_scale);     
                                                       
    if(a_mm_limits.y + (a_mm_limits.h / 2) > (a_b_rlimit / a_mm_scale))  
        a_mm_limits.y = (a_b_rlimit / a_mm_scale) - (a_mm_limits.w / 2);
          
    if(a_mm_limits.x < (a_l_rlimit / a_mm_scale))
        a_mm_limits.x = (a_l_rlimit / a_mm_scale);
          
    if(a_mm_limits.x + (a_mm_limits.w / 2) > (a_r_rlimit / a_mm_scale)) 
        a_mm_limits.x = (a_r_rlimit / a_mm_scale) - (a_mm_limits.w / 2);
        
    a_view_rect->SetPosition(vx - a_mm_limits.x, vy - a_mm_limits.y);

    RefreshBuffer();                
    Refresh();    
}

// update the view of the minimap according to scene view position
void MiniMapPanel::UpdateView(Sint16 x, Sint16 y)
{
    bool need_buf_refresh = false;
    
    Point rect_p;
    
    // update view rect position
    rect_p.x = (x / a_mm_scale) - a_mm_limits.x;
    rect_p.y = (y / a_mm_scale) - a_mm_limits.y;
                          
    // update minimap limits
    if(rect_p.y < 0 || (rect_p.y + a_view_rect->GetH()) > a_mm_limits.h)
    {      
        a_mm_limits.y = (y / a_mm_scale) - a_view_rect->GetY();
        rect_p.y = (y / a_mm_scale) - a_mm_limits.y;
                        
        need_buf_refresh = true;        
    }
    if(rect_p.x < 0 || (rect_p.x + a_view_rect->GetW()) > a_mm_limits.w)
    {
        a_mm_limits.x = (x / a_mm_scale) - a_view_rect->GetX();
        rect_p.x = (x / a_mm_scale) - a_mm_limits.x;
           
        need_buf_refresh = true;        
    }
    
    a_view_rect->SetPosition(rect_p);
    
    if(need_buf_refresh)
        RefreshBuffer();

    // refresh panel
    blitter->Copy(*a_buffer, *a_panel);
    blitter->Blit(*a_view_rect, *a_panel); 
}
