#ifndef CLASS_MINI_MAP_PANEL
#define CLASS_MINI_MAP_PANEL

#include "graphic/BaseSurface.h"
#include "graphic/Blitter.h"
#include "graphic/Image.h"
#include "graphic/Surface.h"

#include "DataTypes.h"
#include "IsoMiniMap.h"

#include <iostream>

/// this class implements a panel that contains a minimap with
/// its view rectangle
class MiniMapPanel
{
	private:
     /// the surface that made the panel
     graphic::Surface * a_panel;
     /// buffer used to store the minimap image 
     graphic::Surface * a_buffer;
     /// the rectangular image that limits the view of the scene
     graphic::Image * a_view_rect;
     /// current visible limits of the minimap
     SDL_Rect a_mm_limits;
     
     SDL_Color a_back_color;
     
     /// pointer to the minimap
     IsoMiniMap * a_mm;
     
     /* current visible limits of the renderer */
     /// upper render limit
     Sint16 a_u_rlimit;
     /// bottom render limit
     Sint16 a_b_rlimit;
     /// left render limit
     Sint16 a_l_rlimit;
     /// right render limit
     Sint16 a_r_rlimit;
     
     /// scale of the minimap
     int a_mm_scale;
               
     /// refresh the buffer containing the minimap image
     void RefreshBuffer()
     {   
         a_buffer->Fill(a_back_color);
         a_mm->BlitLayer(*a_buffer, &a_mm_limits); 
     };

	public:
     /// constructor
     MiniMapPanel(IsoMiniMap * mm, graphic::Image * view_rect, SDL_Rect mm_limits);
     /// destroyer
     ~MiniMapPanel() 
     { 
         delete a_panel;
         delete a_buffer;
     };
     
     /// set renderer limits
     void SetRendererLimits(Sint16 u_limit, Sint16 b_limit, Sint16 l_limit, Sint16 r_limit)
     {
         a_u_rlimit = u_limit;        
         a_b_rlimit = b_limit;
         a_l_rlimit = l_limit;
         a_r_rlimit = r_limit;  
     };
     
     /// set X component of minimap limits
     void SetLimitX(Sint16 x) { a_mm_limits.x = x; };
     /// set Y component of minimap limits
     void SetLimitY(Sint16 y) { a_mm_limits.y = y; };
     /// set W component of minimap limits
     void SetLimitW(Uint16 w) { a_mm_limits.w = w; };
     /// set Y component of minimap limits
     void SetLimitH(Uint16 h) { a_mm_limits.h = h; };
     
     void SetLimit(SDL_Rect * limit) 
     {
         a_mm_limits.x = limit->x;
         a_mm_limits.y = limit->y;
         a_mm_limits.w = limit->w;
         a_mm_limits.h = limit->h;
     };
     
     /// update the view of the minimap according to scene view position
     void UpdateView(Sint16 x, Sint16 y);
     
     /// mouse has been clicked on the panel at (x,y) point
     void MouseClicked(Sint16 x, Sint16 y);
     
     /// refresh the content of the panel
     void Refresh()
     {
         // refresh buffer
         RefreshBuffer();
         
         // refresh panel
         graphic::blitter->Copy(*a_buffer, *a_panel);
         graphic::blitter->Blit(*a_view_rect, *a_panel);         
     };

     /// blit the panel on a dest surface
     void Blit(graphic::BaseSurface * dest) { graphic::blitter->Blit(*a_panel, *dest); };
     
     /// return the view X position for the scene
     Sint16 GetViewX() { return (a_view_rect->GetX() + a_mm_limits.x) * a_mm_scale; };
     /// return the view Y position for the scene
     Sint16 GetViewY() { return (a_view_rect->GetY() + a_mm_limits.y) * a_mm_scale; };
     
     /// return the scale of the minimap
     int GetScale() { return a_mm_scale; };
};

#endif
