#include "Mouse.h"
#include <iostream>

using namespace std;
using namespace graphic;

// Constructor
Mouse::Mouse(vector< graphic::MovableImage * > & pointers)
{
    int x, y;
    SDL_GetMouseState(&x, &y);  
    
    // position on the screen
    a_screen_position = new Point;      
    a_screen_position->x = x;
    a_screen_position->y = y;
    
    // absolute position on the map
    a_abs_position = new Point;
    a_abs_position->x = x;
    a_abs_position->y = y;
    
    // origin of the screen view     
    a_view = new Point;
    a_view->x = 0;
    a_view->y = 0;

    // store pointers images
    int num_imgs = pointers.size();
    
    for(int i= 0; i < num_imgs; i++)
        a_pointers.push_back(pointers[i]);
    
    a_cur_pointer = pointers[0];
    
    a_cur_pointer->SetInitialPosition(x, y);
}

// Destroyer
Mouse::~Mouse()
{
    delete a_abs_position;
    delete a_screen_position;
    delete a_view;
}

// Set the current view values and update position
void Mouse::SetView(Sint16 view_x, Sint16 view_y) 
{ 
    a_view->x = view_x;  
    a_view->y = view_y;
      
    // set absolute position on the map
    a_abs_position->x = a_screen_position->x + a_view->x;
    a_abs_position->y = a_screen_position->y + a_view->y;
}

// Set the position passing just the screen position   
void Mouse::SetPosition(Sint16 x, Sint16 y)
{       
    // set screen position     
    a_screen_position->x = x;
    a_screen_position->y = y;
    
    // set absolute position on the map
    a_abs_position->x = x + a_view->x;
    a_abs_position->y = y + a_view->y;
}

// set the pointer image
void Mouse::SetPointer(int p)
{
    // same pointer, do nothing
    if(a_pointers[p] == a_cur_pointer)
        return ;
    
    // restore screen
    a_cur_pointer->RestoreBackground(graphic::screen);
    a_cur_pointer->UpdateScreen(); 
    
    Sint16 x, y;    
    x = a_cur_pointer->GetX();
    y = a_cur_pointer->GetY();
    
    // change pointer
    a_cur_pointer = a_pointers[p];
    
    // set position and save backg
    a_cur_pointer->SetInitialPosition(x, y);
    a_cur_pointer->SaveBackground(graphic::screen);
}
