#ifndef CLASS_MOUSE
#define CLASS_MOUSE

#include <vector>

#include "graphic/Blitter.h"
#include "graphic/MovableImage.h"
#include "graphic/Screen.h"
#include "DataTypes.h"
#include "Functions.h"

/// mouse management
class Mouse
{
    private:
     /// vector of mouse pointers
     std::vector< graphic::MovableImage * > a_pointers;    
     /// pointer to the image currently used 
     graphic::MovableImage * a_cur_pointer; 

	protected:	
     /// absolute position on the map
     Point * a_abs_position;
     /// position on the screen
     Point * a_screen_position;
     /// origin of the screen view     
     Point * a_view;     
     
	public:
     /// Constructor
     Mouse(std::vector< graphic::MovableImage * > & pointers);
     /// Destroyer
     virtual ~Mouse();
     
     /// Set the current view values
     void virtual SetView(Sint16 view_x, Sint16 view_y);
     /// Set the current view values passing a point
     void virtual SetView(Point view) { SetView(view.x, view.y); };
     
     /// Set the position passing just the screen position   
     void virtual SetPosition(Sint16 x, Sint16 y);
     /// Set the position passing just the point of screen position    
     void virtual SetPosition(Point pos) { SetPosition(pos.x, pos.y); };     
     /// Set the position passing screen position and view position   
     void virtual SetPosition(Sint16 x, Sint16 y, Sint16 view_x, Sint16 view_y) 
     { 
         SetView(view_x, view_y);
         SetPosition(x, y);
     };
     /// Set the position passing screen position as point and view position as point
     void virtual SetPosition(Point pos, Point view) { SetPosition(pos.x, pos.y, view.x, view.y); };
    
     /// return the X screen position
     Sint16 GetX() { return a_screen_position->x; };
     /// return the Y screen position
     Sint16 GetY() { return a_screen_position->y; };
     
     /// return the absolute X position
     Sint16 GetAbsX() { return a_abs_position->x; };
     /// return the absolute Y position
     Sint16 GetAbsY() { return a_abs_position->y; };
     
     /// save the screen under the pointer in the buffer and then blit on screen
     void Blit()
     {
         a_cur_pointer->SaveBackground(graphic::screen);                
         graphic::blitter->Blit(*a_cur_pointer); 
     };
     
     /// restore the background blitting the buffer on screen
     void RestoreBackg() {a_cur_pointer->RestoreBackground(graphic::screen); }
     
     /// restore the background, move and blit
     void UpdateAndBlit()
     {
         a_cur_pointer->RestoreBackground(graphic::screen);                
         a_cur_pointer->SetPosition(a_screen_position->x, a_screen_position->y);                
         a_cur_pointer->SaveBackground(graphic::screen);
         graphic::blitter->Blit(*a_cur_pointer);                                                
         a_cur_pointer->UpdateScreen(); 
     };
     
     /// move and blit without restoring the background
     void MoveAndBlit()
     {        
         a_cur_pointer->SetPosition(a_screen_position->x, a_screen_position->y);                
         a_cur_pointer->SaveBackground(graphic::screen);
         graphic::blitter->Blit(*a_cur_pointer);                                                
         a_cur_pointer->UpdateScreen(); 
     };
     
     /// set the pointer image
     void SetPointer(int p);
};

#endif
