#include "PathHighlighter.h"

using namespace std;
using namespace graphic;

// Constructor
PathHighlighter::PathHighlighter(IsoTilesMap * tm, Image * good, Image * bad, Image * final, string font)
{
    // store rows, cols
    a_num_rows = tm->GetNumRows();
    a_num_cols = tm->GetNumCols(); 
    // store tile dimensions
    a_tile_width = tm->GetTileW();
    a_tile_height = tm->GetTileH();
    
    // store the graphic origin
    a_graphic_origin = new Point;
    a_graphic_origin->x =  tm->GetGraphicOriginX();
    a_graphic_origin->y =  tm->GetGraphicOriginY();
    
    // store higlighters
    a_good_highlighter = good;        
    a_bad_highlighter = bad;        
    a_final_highlighter = final;
        
    // set opacity
    a_good_highlighter->SetOpacity(PATH_TRANS);
    a_bad_highlighter->SetOpacity(PATH_TRANS);
    a_final_highlighter->SetOpacity(PATH_TRANS);
    
    // remaining AP Text
    a_remaining_ap = NULL;
    
    // type of path
    a_good_path = true;

    // default colors
    a_font_good_color = new SDL_Color;
    a_font_good_color->r = 0xFF;
    a_font_good_color->g = 0xFF;
    a_font_good_color->b = 0;
    a_font_good_color->unused = 0;
    
    a_font_bad_color = new SDL_Color;    
    a_font_bad_color->r = 0xFF;
    a_font_bad_color->g = 0xCC;
    a_font_bad_color->b = 0;
    a_font_bad_color->unused = 0;
    
    // store AP font 
    a_font = font;
    // default font size
    a_font_size = RAP_FONT_SIZE;
}

// Destroyer
PathHighlighter::~PathHighlighter()
{
    delete a_graphic_origin;
    delete a_font_good_color;
    delete a_font_bad_color;
}
        
// set the remaining Action Points to show in the last cell of the path
void PathHighlighter::SetRemainingAP(int ap)
{   
    // delete previous surface   
    if(a_remaining_ap != NULL)    
        delete a_remaining_ap;
        
    // good path
    if(ap >= 0)
    {
        a_good_path = true;        
        a_remaining_ap = new Text(ap, *a_font_good_color, a_font, a_font_size);
    }
    // bad path
    else
    {
        a_good_path = false;
        a_remaining_ap = new Text(ap, *a_font_bad_color, a_font, a_font_size);
    }
}
        
// Blit the effect on a surface if it fit the limits rect
void PathHighlighter::BlitLayer(graphic::BaseSurface & dst_surf, SDL_Rect * limits_r)
{
    Sint16 l_limit = limits_r->x ;
    Sint16 r_limit = limits_r->x + limits_r->w;
    Sint16 u_limit = limits_r->y;
    Sint16 b_limit = limits_r->y + limits_r->h;
    
    Point * p = new Point;
    
    // total cells of the path
    int num_cells = a_path->GetPathLen();
    // cells that use the same image
    int num_equal_cells;    
    
    int half_height = a_tile_height / 2;
    
    Image * cell_img;
    
    // good path => final cell is different 
    if(a_good_path)
    {
        cell_img = a_good_highlighter;
        num_equal_cells = num_cells - 1;
    }
    // bad path => all cells are equal
    else
    {        
        cell_img = a_bad_highlighter;
        num_equal_cells = num_cells;
    }
    
    PathNode * node;
    
    // blit equal cells
    for(int i = 0; i < num_equal_cells; i++)
    {
        node = a_path->GetNode(i);        
        
        p->x = (node->GetCol() - node->GetRow()) * a_tile_height + a_graphic_origin->x;
        p->y = (node->GetCol() + node->GetRow()) * half_height + a_graphic_origin->y;
        
        if( (p->x + a_tile_width >= l_limit) && (p->x <= r_limit) && 
            (p->y + a_tile_height >= u_limit) && (p->y <= b_limit))
        {
            //set blit position of the image
            cell_img->SetPosition(p->x - l_limit, p->y - u_limit );
            //blit 
            blitter->Blit(* cell_img, dst_surf);
        }   
    }    
    
    // if good path, blit the final cell
    if(a_good_path)
    {
        node = a_path->GetNode(num_cells - 1);
        
        // final cell
        p->x = (node->GetCol() - node->GetRow()) * a_tile_height + a_graphic_origin->x;
        p->y = (node->GetCol() + node->GetRow()) * half_height + a_graphic_origin->y;
        
        if( (p->x + a_tile_width >= l_limit) && (p->x <= r_limit) && 
            (p->y + a_tile_height >= u_limit) && (p->y <= b_limit))
        {
            //set blit position of the image
            a_final_highlighter->SetPosition(p->x - l_limit, p->y - u_limit );
            //blit 
            blitter->Blit(* a_final_highlighter, dst_surf);
        }
    }
       
    // blit remaining AP on the final cell
    p->x += (a_tile_width - a_remaining_ap->GetW()) / 2;
    p->y += (a_tile_height - a_remaining_ap->GetH()) / 2;
    
    if( (p->x + a_remaining_ap->GetW() >= l_limit) && (p->x <= r_limit) && 
        (p->y + a_remaining_ap->GetH() >= u_limit) && (p->y <= b_limit))
    {
        a_remaining_ap->SetPosition(p->x -l_limit, p->y - u_limit);
        blitter->Blit(* a_remaining_ap, dst_surf);
    }            
}

// Blit the effect on a surface
void PathHighlighter::BlitLayer(graphic::BaseSurface & dst_surf)
{
    
}
