#ifndef CLASS_PATH_HIGHLIGHTER
#define CLASS_PATH_HIGHLIGHTER

#include <string>
#include <vector>

#include "graphic/Image.h"
#include "graphic/Text.h"

#include "IsoTilesMap.h"
#include "DataTypes.h"
#include "Layer.h"
#include "Path.h"
#include "PathNode.h"

#define PATH_TRANS 50
// default font used to show remaining Action Points
#define RAP_FONT "data/font/tahomabd.ttf"
#define RAP_FONT_SIZE 13

/// Manage the graphical path highlighting, this class needs to work on a tiles map
class PathHighlighter : public Layer
{
	private:
        /// number of rows of the map
        int a_num_rows;
        /// number of cols of the map
        int a_num_cols;
        /// height of a tile of the map
        int a_tile_height;
        /// width of a tile of the map
        int a_tile_width;
        /// graphical origin of the [0,0] tile of the map
        Point * a_graphic_origin;
        /// path cells     
        Path * a_path;       
        /// path type TRUE => GOOD / FALSE => BAD
        bool a_good_path;
        
        /// good cell higlighter image
        graphic::Image * a_good_highlighter;
        /// bad cell higlighter image
        graphic::Image * a_bad_highlighter;
        /// final cell higlighter image
        graphic::Image * a_final_highlighter;
        /// remaining Action Points to show in the last cell of the path
        graphic::Text * a_remaining_ap;
        
        /// path of the font used to show remaining AP
        std::string a_font;
        int a_font_size;
        
        /// color of the font when there is a good path
        SDL_Color * a_font_good_color;
        /// color of the font when there is a bad path
        SDL_Color * a_font_bad_color;
        
	public:
        /// Constructor
        PathHighlighter(IsoTilesMap * tm, graphic::Image * good, graphic::Image * bad, 
                        graphic::Image * final, std::string font = RAP_FONT);
        /// Destroyer
        ~PathHighlighter();
        
        /// set the current path
        void SetPath(Path * path) { a_path = path; };
        
        /// set the remaining Action Points to show in the last cell of the path
        void SetRemainingAP(int ap);
        
        /// set the good path font color
        void SetGoodFontColor(SDL_Color color)
        {
            a_font_good_color->r = color.r;
            a_font_good_color->g = color.g;
            a_font_good_color->b = color.b;    
        };
        /// set the bad path font color
        void SetBadFontColor(SDL_Color color)
        {
            a_font_bad_color->r = color.r;
            a_font_bad_color->g = color.g;
            a_font_bad_color->b = color.b;    
        };                
        
        void SetFontSize(int size) { a_font_size = size; };
        
        /// Blit the effect on a surface if it fit the limits rect
        void BlitLayer(graphic::BaseSurface & dst_surf, SDL_Rect * limits_r);
        /// Blit the effect on a surface
        void BlitLayer(graphic::BaseSurface & dst_surf);   
};

#endif
