#include "Pathfind.h"
// #include <iostream>

using namespace std;

// Constructor
Pathfind::Pathfind(IsoTilesMap * tm)
{
    // storing the IsoTilesMap
    a_tm = tm;
          
    // init the membership vector => all elements == 0
    int max_nodes = a_tm->GetNumRows() * a_tm->GetNumCols();
    bitset<2> * flag;            
    for(int i = 0; i < max_nodes; i++)
    {
        flag = new bitset<2>(0);        
        a_membership.push_back(flag);
    }    
}

// Destroyer
Pathfind::~Pathfind()
{
    int max_nodes = a_membership.size();   
         
    for(int i = 0; i < max_nodes; i++)    
        delete a_membership[i];

    a_membership.clear();
}

// Try to make a path from start[row,col] to end[row,col]
Path * Pathfind::MakePath(int start_row, int start_col, int end_row, int end_col, int start_dir)
{                                             
    PathfindNode * node;
    PathfindNode * cur_node;
        
    a_end.row = end_row;
    a_end.col = end_col;
    
    node = new PathfindNode(start_row, start_col, GenNodeId(start_row, start_col));    
    node->SetDirection(start_dir);
    
    // store the first node into OPEN and save the membership
    (a_membership[node->GetId()])->set(OPEN);
    a_open.AddNode(node);
    
    bool path_found = false;
    
    Path * path = new Path;   

    // llok for the end node until there are available nodes
    while(!path_found && !a_open.IsEmpty())
    {
        cur_node = a_open.GetHead();
        (a_membership[cur_node->GetId()])->reset(); 
    
        a_closed.push_back(cur_node);
        (a_membership[cur_node->GetId()])->set(CLOSED);
                              
        if((cur_node->GetRow() == a_end.row) && (cur_node->GetCol() == a_end.col))
            path_found = true;
        else                         
            ExpandNodes(cur_node);
    }
                
    // no path found, return NULL
    if(!path_found)
    {
        CleanPathfindData();
        return NULL;
    }
            
    // create the inverted path using a stack
    stack< PathfindNode * > path_stack;       
    // start from the end node
    node = cur_node;

    while(node)      
    {   
        path_stack.push(node);                    
        node = node->GetFather();
    }       
    
    // create the path using the nodes from the stack
    while(!path_stack.empty())
    {
        node = path_stack.top();
        path_stack.pop();
        
        path->AddNode(node->GetRow(), node->GetCol(), node->GetCost(), node->GetDirection());
    }  
    
    // clean data
    CleanPathfindData();
    
    return path;
}

// expands near nodes from one centered
// checks that nodes are inside the map, walkable and not yet into the closed list
void Pathfind::ExpandNodes(PathfindNode * center)
{
    int c_row = center->GetRow();
    int c_col = center->GetCol();
    
    // NORTH
    if((c_row - 1) >= 0 && 
       (a_tm->IsWalkable(c_row - 1, c_col) || a_tm->IsVisible(c_row - 1, c_col) != VISIBLE) &&
       !(a_membership[GenNodeId(c_row - 1, c_col)])->test(CLOSED))
        GenNode(center, c_row - 1, c_col, DIR_NORTH);
    
    // NORTH-EAST
    if((c_row - 1) >= 0 && (c_col + 1) < a_tm->GetNumCols() && 
       (a_tm->IsWalkable(c_row - 1, c_col + 1) || a_tm->IsVisible(c_row - 1, c_col + 1) != VISIBLE) &&
       !(a_membership[GenNodeId(c_row - 1, c_col + 1)])->test(CLOSED))
        GenNode(center, c_row - 1, c_col + 1 , DIR_NORTH_EAST);
    
    // EAST
    if((c_col + 1) < a_tm->GetNumCols() &&
       (a_tm->IsWalkable(c_row, c_col + 1) || a_tm->IsVisible(c_row, c_col + 1) != VISIBLE) &&
       !(a_membership[GenNodeId(c_row, c_col + 1)])->test(CLOSED))
        GenNode(center, c_row, c_col + 1 , DIR_EAST);
    
    // SOUTH-EAST
    if((c_row + 1) < a_tm->GetNumRows() && (c_col + 1) < a_tm->GetNumCols() &&
       (a_tm->IsWalkable(c_row + 1, c_col + 1) || a_tm->IsVisible(c_row + 1, c_col + 1) != VISIBLE) &&
       !(a_membership[GenNodeId(c_row + 1, c_col + 1)])->test(CLOSED))
        GenNode(center, c_row + 1, c_col + 1 , DIR_SOUTH_EAST);
    
    // SOUTH
    if((c_row + 1) < a_tm->GetNumRows() &&
       (a_tm->IsWalkable(c_row + 1, c_col) || a_tm->IsVisible(c_row + 1, c_col) != VISIBLE) &&
       !(a_membership[GenNodeId(c_row + 1, c_col)])->test(CLOSED))
        GenNode(center, c_row + 1, c_col , DIR_SOUTH);
        
    // SOUTH-WEST
    if((c_row + 1) < a_tm->GetNumRows() && (c_col - 1) >= 0 &&
       (a_tm->IsWalkable(c_row + 1, c_col - 1) || a_tm->IsVisible(c_row + 1, c_col - 1) != VISIBLE) &&
       !(a_membership[GenNodeId(c_row + 1, c_col - 1)])->test(CLOSED))
        GenNode(center, c_row + 1, c_col - 1 , DIR_SOUTH_WEST);
    
    // WEST
    if((c_col - 1) >= 0 &&
       (a_tm->IsWalkable(c_row, c_col - 1) || a_tm->IsVisible(c_row, c_col - 1) != VISIBLE) &&
       !(a_membership[GenNodeId(c_row, c_col - 1)])->test(CLOSED))    
        GenNode(center, c_row, c_col - 1 , DIR_WEST);    
    
    // NORTH-WEST
    if((c_row - 1) >= 0 && (c_col - 1) >= 0 &&
       (a_tm->IsWalkable(c_row - 1, c_col - 1) || a_tm->IsVisible(c_row - 1, c_col - 1) != VISIBLE) &&
       !(a_membership[GenNodeId(c_row - 1, c_col - 1)])->test(CLOSED))
        GenNode(center, c_row - 1, c_col - 1 , DIR_NORTH_WEST);    
}

// generate a node from the passed one, in according with the passed direction
void Pathfind::GenNode(PathfindNode * from,int new_row, int new_col, int direction)
{
    int g;
    int id = GenNodeId(new_row, new_col);

    
    PathfindNode * node;
                        
    // compute cost of the node
    if((direction % 2) == 0)
        g = ORTHO_COST;
    else
        g = DIAGONAL_COST;
                    
    if(direction != from->GetDirection())
        g += ROTATION_COST;    
    
    // node not present in any list
    if(!(a_membership[id])->test(OPEN))
    {                            
        node = new PathfindNode(new_row, new_col, GenNodeId(new_row, new_col));         
        
        node->SetDirection(direction);        
        node->SetCost(g);
        // set F = G + H
        node->SetScores(g + from->GetG(), ComputeHeuristics(node));
        // set the father of the node
        node->SetFather(from);  
    
        (a_membership[id])->set(OPEN);
        a_open.AddNode(node);                 
    }
    // node is inside the open
    else 
    {        
        node = a_open.SearchNode(id);

        if(node->GetG() > (g + from->GetG()))       
        {
            node->SetRow(new_row);
            node->SetCol(new_col);            
            node->SetDirection(direction);            
            node->SetCost(g);            
            node->SetScores(g + from->GetG(), ComputeHeuristics(node));            
            node->SetFather(from);  
        }                        
    }    
}

// compute the heuristics value of the remaining path from a passed node
// in this case DIAGONAL DISTANCE
int Pathfind::ComputeHeuristics(PathfindNode * node)
{
    int h1_diag = abs(node->GetRow() - a_end.row);
    int h2_diag = abs(node->GetCol() - a_end.col);
            
    int h_diag = ((h1_diag < h2_diag) ? h1_diag : h2_diag);
    
    int h_straight = (abs(node->GetRow() - a_end.row) + abs(node->GetCol() - a_end.col));
                    
    return ((DIAGONAL_COST * h_diag) + (ORTHO_COST * (h_straight - (2 * h_diag))));
}

// erase unused resource
void Pathfind::CleanPathfindData()
{
    // reset membership elements
    int num_elem = a_membership.size();
    
    for(int i = 0; i < num_elem; i++)
        a_membership[i]->reset();
    
    // clear OPEN
    a_open.Clear();
    
    // clear CLOSED
    num_elem = a_closed.size();
    
    for(int i = 0; i < num_elem; i++)
        delete a_closed[i];
    
    a_closed.clear();    
}
