#ifndef CLASS_PATHFIND_NODE
#define CLASS_PATHFIND_NODE

#include <functional>

#include "PathNode.h"

/// a node generated and used by the pathfind to make the path
class PathfindNode : public PathNode
{
	private:
     /// unique id of the node
     int a_id;
     /// prediceted full path cost passing for this node ( a_f = a_g + a_h )
     int a_f;
     /// path cost until this node
     int a_g;
     /// predicted path cost from this node to the end
     int a_h;
     /// pointer to the father
     PathfindNode * a_father;

	public:
     /// constructor, inits attributes
     PathfindNode(int row, int col, int id) : PathNode(row, col)
     {
         a_id = id;         
         a_f = a_g = a_h = 0;         
         a_father = NULL;
     }
     /// Destroyer
     ~PathfindNode() { };

     /// set father pointer
     void SetFather(PathfindNode * father) { a_father = father; };
     /// set f, g, h scores (f = g + h)
     void SetScores(int g, int h)
     {
         a_f = g + h;
         a_g = g;
         a_h = h;   
     }

     /// return F value, predicted full path cost passing for this node (F = G + H)
     int GetF() { return a_f; };
     /// return G value, path cost until this node
     int GetG() { return a_g; };
     /// return H value, predicted path cost from this node to the end
     int GetH() { return a_h; };

     /// return the node id
     int GetId() { return a_id; };
     
     /// return the father of the node
     PathfindNode * GetFather() { return a_father; };
};

/// binary predicate, used to compare the F value of two nodes
struct GreaterPathfindNode : public std::binary_function<PathfindNode *, PathfindNode *, bool>
{
    bool operator()(PathfindNode * n1, PathfindNode * n2) 
    {  
        return ((n1->GetF()) > (n2->GetF()));
    }
};

#endif
