#include "SelectionData.h"

using namespace graphic;
using namespace std;

// constructor
SelectionData::SelectionData(vector < Image * > images)
{
    int num_imgs = images.size();
    
    // store images pointers
    for(int i = 0; i < num_imgs; i++)
        a_images.push_back(images[i]);
            
    // no obj selected by default
    a_sel_elem = NULL;
}

// Blit the effect on a surface if it fits the limits rect
void SelectionData::BlitLayer(BaseSurface & dst_surf, SDL_Rect * limits_r)
{
    if(a_sel_elem == NULL)
        return ;
    
    Sint16 l_limit = limits_r->x ;
    Sint16 u_limit = limits_r->y;
    
    Image * obj_img = a_sel_elem->GetImage();
        
    // temporary Point
    Point * p = new Point; 
     
    p->x = a_sel_elem->GetX() + 
           ((obj_img->GetVisibleX() + obj_img->GetVisibleW() - a_images[BAR_BORDER_IMG]->GetW())/2) ;            
    p->y = a_sel_elem->GetY() + obj_img->GetVisibleY() - BORDER_V_OFFSET;          
    
    // check if the bar is inside the scene
    if(p->x + a_images[BAR_BORDER_IMG]->GetW() >= l_limit && p->y + a_images[BAR_BORDER_IMG]->GetH()>=  u_limit)
    {
        a_images[BAR_BORDER_IMG]->SetPosition(p->x - l_limit, p->y - u_limit);
        
        //blit border   
        blitter->Blit(*(a_images[BAR_BORDER_IMG]), dst_surf);
        
                
        // blit all bad bar under the good one
        a_images[BAR_BAD_IMG]->SetPosition(p->x + BAR_H_OFFSET - l_limit, 
                                           p->y + BAR_V_OFFSET - u_limit);
        
        blitter->Blit(*(a_images[BAR_BAD_IMG]), dst_surf);
        
        
        // calculate energy scale 
        int scale = a_sel_elem->GetMaxStat(STAT_ENERGY) / a_images[BAR_GOOD_IMG]->GetW();         
        
        if(!scale)
            scale = 1;
        
        SDL_Rect * fill_r = new SDL_Rect;
        SDL_Rect * dest_r = new SDL_Rect;
                
        // blit part of the good bar according to element energy
        fill_r->x = 0;
        fill_r->y = 0;
        fill_r->w = a_sel_elem->GetCurStat(STAT_ENERGY) / scale;          
        fill_r->h = a_images[BAR_GOOD_IMG]->GetH(); 
        
        dest_r->x = p->x + BAR_H_OFFSET - l_limit;
        dest_r->y = p->y + BAR_V_OFFSET - u_limit;            

        if(fill_r->w > 0)
            blitter->Blit(*(a_images[BAR_GOOD_IMG]), *fill_r, dst_surf, *dest_r);            

        delete fill_r;
        delete dest_r;
    }
    
    delete p;
}

// Blit the effect on a surface
void SelectionData::BlitLayer(BaseSurface & dst_surf)
{
    if(a_sel_elem == NULL)
        return ;
}
