#ifndef CLASS_TILE
#define CLASS_TILE

#include <bitset>
#include "Object.h"
#include "graphic/Image.h"

#define VIEW_MODES 3

#define NOT_VISIBLE 0
#define PARTIALLY_VISIBLE 1
#define VISIBLE 2

/// A tile of isometric map
class Tile: public Object 
{
	private:
		/// flag that says if a cell is walkable or not
		bool a_walkable;
		/// flag that says if a cell is visible or not
		std::bitset<VIEW_MODES> a_visible;
		/// pointer to the Image stored into the vector of tiles of the map
		graphic::Image * a_img;
		
	public:
		/// constructor, just load the image and set default values
		Tile(graphic::Image * img);
		/// destroyer
		~Tile() {};
		
		/// Set the a_walkable value
		void SetWalkable(bool w) { a_walkable = w; };
		/// return true if the tile is walkable, not otherwise
		bool IsWalkable() { return a_walkable; };
		
		/// Set the visibility value
		void SetVisible(int v) 
		{ 
			a_visible.reset();
			a_visible.set(v); 
		};
		
		/// return visibility value
		int IsVisible();
		
		/// get image pointer of the tile
		graphic::Image * GetImage() { return a_img; };
};

#endif
