#include "TileSet.h"

#include <iostream>

using namespace std;
using namespace graphic;

// this constructor use a vector of strings to create the tileset
// set trans as transparent color
TileSet::TileSet(vector < string > files, SDL_Color * trans)
{
	int num_img = files.size();
	
	for(int i = 0; i < num_img; i++)
		AddImg(files[i], trans);
	
	a_tile_width = a_images[0]->GetW();
	a_tile_height = a_images[0]->GetH();
}	

// this constructor read all the images from a passed directory
// set trans as transparent color
TileSet::TileSet(string dir, SDL_Color * trans)
{
	AddDirImg(dir, trans);
	
	a_tile_width = a_images[0]->GetW();
	a_tile_height = a_images[0]->GetH();
}

// delete all images
TileSet::~TileSet()
{
	int num_img = a_images.size();
	
	for(int i = 0; i < num_img; i++)
	{
		delete a_images[i];
		delete a_files[i];
	}
}

// add just an image to the tile set
// set trans as transparent color
void TileSet::AddImg(string file, SDL_Color * trans)
{
	Image * img;
	string * str;
	
	img = new Image(file, *trans);
	str = new string(file);
	
	a_images.push_back(img);
	a_files.push_back(str);
}

// add all images contained in a passed directory
// set trans as transparent color
void TileSet::AddDirImg(string dir, SDL_Color * trans)
{
	DIR * dir_p;
 	struct dirent * dir_file;
	string * file;
	string * ext;
	Image * img;
	
	if(dir[dir.length() - 1] != '/')
		dir.append("/");
	
	// open the directory
	dir_p = opendir(dir.c_str());
	
	// get files .png from it
	while((dir_file = readdir(dir_p)))
	{
		// file path
		file = new string(dir_file->d_name);
		
		if((*file) == "." || (*file) == "..")
				continue;
			
		// check file extension
		if(file->length() > EXT_LEN)
		{
			// get file extension
			ext = new string(file->substr(file->length() - EXT_LEN));
			
			// allow .PNG
			if((*ext) != "png")
			{
				delete ext;
				
				continue;
			}
			delete ext;
		}
		else
			continue;
			
		a_files.push_back(file);
	}
	
	// sort files
	sort(a_files.begin(), a_files.end(), LessStrPointer());
	
	// add the path
	for(unsigned int i = 0; i < a_files.size(); i++)
		(*a_files[i]) = dir + (*a_files[i]);
		
	// create and store images
	for(unsigned int i = 0; i < a_files.size(); i++)
	{
		if(trans != NULL)
			img = new Image((*a_files[i]), *trans);
		else
			img = new Image((*a_files[i]));
		
		a_images.push_back(img);
	}
	
	// close dir
	closedir(dir_p);	
}

// remove the index image
void TileSet::RemImg(int index)
{
	// take titerators
	vector < graphic::Image * >::iterator img_to_del;
	vector < string * >::iterator file_to_del;
	
	// iterators point to index element
	img_to_del = a_images.begin() + index;
	file_to_del = a_files.begin() + index;
	
	// deallocate memory
	delete *(img_to_del);
	delete *(file_to_del);
	
	// erase elements from the vector
	a_images.erase(img_to_del);
	a_files.erase(file_to_del);
}

// remove an image range from start (included) to end (excluded)
void TileSet::RemImgRange(int start, int end)
{
	// take iterators
	vector < graphic::Image * >::iterator img_to_del;
	vector < string * >::iterator file_to_del;
		
	// iterators point to first element to erase
	img_to_del = a_images.begin() + start;
	file_to_del = a_files.begin() + start;
	
	for(int i = start; i < end; i++)
	{
		// deallocate memory
		delete *(img_to_del);
		delete *(file_to_del);
	
		// erease elements from the vector
		a_images.erase(img_to_del);
		a_files.erase(file_to_del);
	}
}
