#include "BaseSurface.h"
#include <iostream>

using namespace std;

// Fill the a_surface with a color
void graphic::BaseSurface::Fill(SDL_Color & color)
{
	// map fill color
	Uint32 fill_color = SDL_MapRGB(a_surface->format, color.r, color.g, color.b);
	// fill
	SDL_FillRect(a_surface, NULL, fill_color);
}

// Fill part of a surface with a color
void graphic::BaseSurface::Fill(SDL_Color & color, SDL_Rect * f_rect)
{
	// use a temporary rectangle to avoid data loss
	SDL_Rect * rect = new SDL_Rect;
	rect->x = f_rect->x;
	rect->y = f_rect->y;
	rect->w = f_rect->w;
	rect->h = f_rect->h;

	// map fill color
	Uint32 fill_color = SDL_MapRGB(a_surface->format, color.r, color.g, color.b);
	// fill
	SDL_FillRect(a_surface, rect, fill_color);

	delete rect;
}
		
// Get color of a pixel - NOTE: NEED Lock()/Unlock()
Uint32 graphic::BaseSurface::GetPixelColor(Uint16 x, Uint16 y)
{
	int bpp = a_surface->format->BytesPerPixel;
	
	// p is the address to the pixel we want to retrieve
	Uint8 *p = (Uint8 *)a_surface->pixels + y * a_surface->pitch + x * bpp;

	switch(bpp) 
	{
		case 1:
			return *p;

		case 2:
			return *(Uint16 *)p;

		case 3:
			if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
				return p[0] << 16 | p[1] << 8 | p[2];
			else
				return p[0] | p[1] << 8 | p[2] << 16;

		case 4:
			return *(Uint32 *)p;

		default:
			return 0;       
	}
}
