#ifndef CLASS_BASE_SURFACE
#define CLASS_BASE_SURFACE

#include <SDL.h>
#include "../DataTypes.h"

namespace graphic
{

/// This is a minimal, BaseSurface, the super class of Surface and Screen
class BaseSurface
{
	protected:
		/// rectangle that contains the surface 
		SDL_Rect * a_rectangle;
		/// SDL surface
		SDL_Surface * a_surface;
	
	public:
		/// base constructor
		BaseSurface() { a_rectangle = new SDL_Rect; };
		/// destroyer
		virtual ~BaseSurface() { delete a_rectangle; };
		
		/// Get Surface X position (top left corner)
		Sint16 GetX() { return a_rectangle->x; };
		/// Get Surface Y position (top left corner)
		Sint16 GetY() { return a_rectangle->y; };
		/// Get Surface width
		Uint16 GetW() { return a_rectangle->w; };
		/// Get Surface height
		Uint16 GetH() { return a_rectangle->h; };

		/// Fill the surface with a color
		void Fill(SDL_Color & color);
		/// Fill part of a surface with a color
		void Fill(SDL_Color & color, SDL_Rect * f_rect);
		
        /// Get color of a pixel - NOTE: NEED Lock()/Unlock()
		Uint32 GetPixelColor(Uint16 x, Uint16 y);  
        /// Get color of a pixel
        Uint32 GetPixelColorLocked(Uint16 x, Uint16 y) 
        {
            Lock();
            Uint32 c = GetPixelColor(x, y);
            Unlock();
            
            return c;
        }
		
		/// Lock the surface
		void Lock() 
		{ 
			if(SDL_MUSTLOCK(a_surface))
				SDL_LockSurface(a_surface);
		};
		/// unlock the surface
		void Unlock() 
		{ 
			if(SDL_MUSTLOCK(a_surface))
				SDL_UnlockSurface(a_surface);
		};
  
        /// set R,G,B,A values of a pixel at (x,y) - NOTE: NEED Lock()/Unlock()
        void GetRGBA(Uint16 x, Uint16 y, Uint8 *r, Uint8 *g, Uint8 *b, Uint8 *a)
        {
            SDL_GetRGBA(GetPixelColor(x, y), a_surface->format, r, g, b, a);
        };        
        /// return Alpha of a pixel at (x,y) - NOTE: NEED Lock()/Unlock()
        Uint8 GetAlpha(Uint16 x, Uint16 y)
        {
            Uint8 r, g, b, a;       
            SDL_GetRGBA(GetPixelColor(x, y), a_surface->format, &r, &g, &b, &a);    
            return a;
        };
        
        /// set R,G,B,A values of a pixel at (x,y)
        void GetRGBALocked(Uint16 x, Uint16 y, Uint8 *r, Uint8 *g, Uint8 *b, Uint8 *a)
        {
            SDL_GetRGBA(GetPixelColorLocked(x, y), a_surface->format, r, g, b, a);
        };        
        /// return Alpha of a pixel at (x,y)
        Uint8 GetAlphaLocked(Uint16 x, Uint16 y)
        {
            Uint8 r, g, b, a;           
            SDL_GetRGBA(GetPixelColorLocked(x, y), a_surface->format, &r, &g, &b, &a);        
            return a;
        };
                  
        SDL_PixelFormat * GetFormat() { return a_surface->format; }; 
		
		/// SurfaceFriend permits Blitters access to data
		friend class SurfaceFriend;
};

}

#endif
