#ifndef CLASS_BLITTER
#define CLASS_BLITTER

#include <SDL.h>
#include "Screen.h"
#include "BaseSurface.h"
#include "SurfaceFriend.h"

namespace graphic
{
	
/// general blitting interface
class Blitter
{
	protected:
		/// friend of BaseSurface used to work with the SDL surface
		SurfaceFriend * a_sf;
		
	public:
		/// base contructor
		Blitter() { a_sf = new SurfaceFriend; };
		/// destroyer
		virtual ~Blitter() { delete a_sf; };
		
		// NULL -> NULL
		/// Copy an entire surface on screen (must have the same size)
		virtual void Copy(BaseSurface & src_surface) = 0;
		/// Copy an entire surface on another one of the same size
		virtual void Copy(BaseSurface & src_surface, BaseSurface & dst_surface) = 0;
		
		// NULL -> Rect
		/// Blit an entire surface on screen
		virtual void Blit(BaseSurface & src_surface) = 0;
		/// Blit an entire surface on another one
		virtual void Blit(BaseSurface & src_surface, BaseSurface & dst_surface) = 0;
		
		// Rect -> NULL
		/// Blit part of a surface on screen
		virtual void Blit(BaseSurface & src_surface, SDL_Rect & src_rect) = 0;
		/// Blit part of a surface on another one
		virtual void Blit(BaseSurface & src_surface, SDL_Rect & src_rect, 
						  BaseSurface & dst_surface) = 0;
		
		// Rect -> Rect
		/// Blit part of a surface on part of screen
		virtual void Blit(BaseSurface & src_surface, SDL_Rect & src_rect, 
						  SDL_Rect & dst_rect) = 0;
		/// Blit part of a surface on part of another one
		virtual void Blit(BaseSurface & src_surface, SDL_Rect & src_rect, 
					 	  BaseSurface & dst_surface, SDL_Rect & dst_rect) = 0;
};

/// global pointer
extern Blitter * blitter;

}

#endif
