#include "Image.h"

using namespace std;

// Base constructor, Create a new image
graphic::Image::Image(string file)
{
	// Temporary surface
	SDL_Surface *temp;
	
	// Load image
	temp = IMG_Load(file.c_str());
	
	// ERROR
	if(temp == NULL)
	{
        cout << "[ERROR] Image: " << file << " [ERROR]" << endl;
        cout << IMG_GetError() << endl;
       
        // kill the program
        exit(0);
	}

    SDL_SetAlpha(temp , SDL_SRCALPHA | SDL_RLEACCEL , SDL_ALPHA_OPAQUE);
	a_surface = SDL_DisplayFormat(temp);

	// Setting default position
	a_rectangle->x = 0;
	a_rectangle->y = 0;
	// Setting image dimensions
	a_rectangle->w = a_surface->w;
	a_rectangle->h = a_surface->h;
	
	// no trans color
	a_use_trans_color = false;
	a_trans_color = 0;

	// Free temporary surface
	SDL_FreeSurface(temp);
 
    a_vis_rect = new SDL_Rect;
    
    a_vis_rect->x = 0;
    a_vis_rect->y = 0;
    a_vis_rect->w = a_rectangle->w - 1;
    a_vis_rect->h = a_rectangle->h - 1;
}

// Create a new image setting a color as transparent
graphic::Image::Image(string file, SDL_Color trans_color)
{
	// Temporary surface
	SDL_Surface *temp;
	Uint32 colorkey;
	
	// Load image
	temp = IMG_Load(file.c_str());
 
    // ERROR
    if(temp == NULL)
    {
        cout << "[ERROR] Image: " << file << " [ERROR]" << endl;
        cout << IMG_GetError() << endl;
       
        // kill the program
        exit(0);
    }

	a_surface = SDL_DisplayFormat(temp);

	// Setting default position
	a_rectangle->x = 0;
	a_rectangle->y = 0;
	// Setting image dimensions
	a_rectangle->w = a_surface->w;
	a_rectangle->h = a_surface->h;
    
	// Set colorkey as transparent
	colorkey = SDL_MapRGB(a_surface->format, trans_color.r, trans_color.g, trans_color.b);
    SDL_SetColorKey(a_surface, SDL_SRCCOLORKEY|SDL_RLEACCEL, colorkey);

	// set trans color
	a_use_trans_color = true;
	a_trans_color = SDL_MapRGB(a_surface->format, trans_color.r, trans_color.g, trans_color.b);

	// Free temporary surface
	SDL_FreeSurface(temp);
 
    a_vis_rect = new SDL_Rect;
                
    int first_x = a_surface->w - 1;
    int first_y = a_surface->h - 1;    
    int last_x = 0;
    int last_y = 0;
    
    Lock();
            
    for(int x = 0; x < a_surface->w; x++)
    {
        for(int y = 0; y < a_surface->h; y++)
        {
            // opaque color
            if(GetPixelColor(x, y) != a_trans_color)
            {
                if(x < first_x)   
                    first_x = x;
                if(x > last_x)
                    last_x = x;
                
                if(y < first_y)   
                    first_y = y;
                if(y > last_y)
                    last_y = y;                
            }
            
        }
    }
    
    Unlock();
    
    a_vis_rect->x = first_x;
    a_vis_rect->y = first_y;
    a_vis_rect->w = last_x;
    a_vis_rect->h = last_y;
}
