#ifndef CLASS_IMAGE
#define CLASS_IMAGE

#include <string>
#include <iostream>
#include <SDL_image.h>
#include "Surface.h"

#define NO_TRANS_COLOR -1
#define MISSING_IMG "data/img/no_img.png"

namespace graphic
{
/// A surface loaded from an image file
class Image : public virtual Surface
{
	private:
		/// mapped transparent color
		Uint32 a_trans_color;
		/// flag that shows if image use transparent color or not
		bool a_use_trans_color;
  
    protected:
        SDL_Rect * a_vis_rect;
	
	public:
        /// Empty constructor
        Image() { a_use_trans_color = false; };

		/// Base constructor, Create a new image
		Image(std::string file);
		/// Create a new image setting a color as transparent
		Image(std::string file, SDL_Color trans_color);
		/// Destroyer
        ~Image() { delete a_vis_rect; };
		
		/// return true if image use transparent color
		bool UseTransColor() { return a_use_trans_color; };
		/// return the transparent color 
		Uint32 GetTransColor() { return a_trans_color; };
  
        /// return true if the pixel at x,y is transparent, false otherwise
        virtual bool IsTrans(Uint16 x, Uint16 y)
        {   
            if(GetPixelColorLocked(x, y) == a_trans_color)
                return true;
            else
                return false;
        };      
            
        /// Get Surface X position (top left corner)
        Sint16 GetVisibleX() { return a_vis_rect->x; };
        /// Get Surface Y position (top left corner)
        Sint16 GetVisibleY() { return a_vis_rect->y; };
        /// Get Surface width
        Uint16 GetVisibleW() { return a_vis_rect->w; };
        /// Get Surface height
        Uint16 GetVisibleH() { return a_vis_rect->h; };
};

}
#endif
