#include "MovableSurface.h"
#include <iostream>

using namespace std;

// A new, not inizialized, surface 
graphic::MovableSurface::MovableSurface(Uint16 w, Uint16 h, Uint32 flags, Uint16 bpp): Surface(w,h,flags,bpp)
{
	// create backup surface
	a_bk_surface = new Surface(w, h, flags, bpp);
}

// Set the INITIAL position of the upper left Point of the surface (x,y) passing a Point
void graphic::MovableSurface::SetInitialPosition(Point position)
{
	// setting rectangle position
	a_rectangle->x = position.x;
	a_rectangle->y = position.y;
	
	// setting backup rectangle position
	a_bk_surface->SetPosition(position);
}

// Set the INITIAL position of the upper left Point of the surface (x,y) passing x and y values
void graphic::MovableSurface::SetInitialPosition(Sint16 x, Sint16 y)
{
	// setting rectangle position
	a_rectangle->x = x;
	a_rectangle->y = y;

	// setting backup rectangle position
	a_bk_surface->SetPosition(x, y);
}

// Save background of Image before a blit
void graphic::MovableSurface::SaveBackground(BaseSurface * src_surf)
{
	// using a temporary rectangle to preserve data integrity
	SDL_Rect * temp_r = new SDL_Rect;

	temp_r->x = a_rectangle->x;
	temp_r->y = a_rectangle->y;
	temp_r->w = a_rectangle->w;
	temp_r->h = a_rectangle->h;

	// blit from a src_surface to a_bk_surface	
	blitter->Blit(*src_surf, *temp_r, *a_bk_surface);

	delete temp_r; 
}

// Update altered rectangle of screen
void graphic::MovableSurface::UpdateScreen()
{
	// backup rectangle
	SDL_Rect * bk_r = new SDL_Rect;
	// update rectangle
	SDL_Rect * update_r = new SDL_Rect;
	
	bk_r->x = a_bk_surface->GetX();	
	bk_r->y = a_bk_surface->GetY();
	bk_r->w = a_bk_surface->GetW();
	bk_r->h = a_bk_surface->GetH();
	
	//cout << "bk rect (" << bk_r->x << "," << bk_r->y << ") -> " << bk_r->w << "," << bk_r->h << endl;  
	//cout << "rect (" << a_rectangle->x << "," << a_rectangle->y << ") -> " << a_rectangle->w << "," << a_rectangle->h << endl;  

	// get the minimum X,Y
	update_r->x = (a_rectangle->x < bk_r->x) ? a_rectangle->x : bk_r->x;
	update_r->y = (a_rectangle->y < bk_r->y) ? a_rectangle->y : bk_r->y;

	// calculate the movement along X axis
	if(a_rectangle->x == bk_r->x)
		update_r->w = a_rectangle->w;
	else
		update_r->w = abs(a_rectangle->x - bk_r->x) + a_rectangle->w;
	


	// calculate the movement along Y axis
	if(a_rectangle->y == bk_r->y)
		update_r->h = a_rectangle->h;
	else
		update_r->h = abs(a_rectangle->y - bk_r->y) + a_rectangle->h;
	
	//cout << "update rect (" << update_r->x << "," << update_r->y << ") -> " << update_r->w << "," << update_r->h << endl;  

	// refresh screen
	screen->UpdateRect(update_r);
	
	

	// update bk_surface position
	a_bk_surface->SetPosition(a_rectangle->x, a_rectangle->y);

	delete bk_r;
	delete update_r;
}

// Move image of mov pixels along X and Y axis
void graphic::MovableSurface::Move(Sint16 mov_x, Sint16 mov_y)
{	
	// update current surface rectangle
	a_rectangle->x += mov_x; 
	a_rectangle->y += mov_y; 
}
