#ifndef CLASS_MOVABLE_SURFACE 
#define CLASS_MOVABLE_SURFACE

#include <cstdlib>
#include "Surface.h"
#include "Blitter.h"

namespace graphic
{
	
/// A surface usable for animations
class MovableSurface : public virtual Surface
{
	protected:
		/// background surface, this save the background from the blit 
		Surface * a_bk_surface;

	public:
        /// empty constructor
        MovableSurface() {}; 
		/// A new, not inizialized, surface 
		MovableSurface(Uint16 w, Uint16 h, Uint32 flags  = SDL_SWSURFACE, Uint16 bpp = screen->GetBitPP()); 
		/// Destructor
		virtual ~MovableSurface() { delete a_bk_surface; };
		
		/// Set INITIAL position of the upper left Point of the surface (x,y)
		/// passing a Point
		void SetInitialPosition(Point position);
		/// Set INITIAL position of the upper left Point of the surface (x,y) 
		/// passing x and y values
		void SetInitialPosition(Sint16 x, Sint16 y);
        /// Set the position of the upper left Point of the surface (x,y) passing a Point
        void SetPosition(Point position)
        {
            a_rectangle->x = position.x;
            a_rectangle->y = position.y;
        }
        /// Set the position of the upper left Point of the surface (x,y) passing x and y values
        void SetPosition(Sint16 x, Sint16 y)
        {
            a_rectangle->x = x;
            a_rectangle->y = y;
        }

		// Move image of mov pixels along X and Y axis
		void Move(Sint16 mov_x, Sint16 mov_y);
		/// Move image of mov pixels along X axis
		void MoveX(Sint16 mov) { a_rectangle->x += mov; };
		/// Move image of mov pixels along Y axis
		void MoveY(Sint16 mov) { a_rectangle->y += mov; };

		/// Save background of Image before a blit
		void SaveBackground(BaseSurface * src_surf);
		/// Restore a saved background of Image after a blit and update a_bk_rectangle
		void RestoreBackground(BaseSurface * dst_surf) { blitter->Blit(*a_bk_surface, *dst_surf); };

		/// Update altered rectangle of screen
		void UpdateScreen();
};

}
#endif
