#include "Screen.h"
#include <iostream>

using namespace std;

// Init screen surface
graphic::Screen::Screen(int w, int h, Uint32 flags)
{
	// Saving flags value
	a_flags = flags;
			
	a_rectangle->x = 0;
	a_rectangle->y = 0;
	// Setting screen dimensions
	a_rectangle->w = w;
	a_rectangle->h = h;

	// Setting screen mode
	a_surface = SDL_SetVideoMode(a_rectangle->w, a_rectangle->h, 0, a_flags); 
	
	a_backup = NULL;
}
	

// Set screen flags
void graphic::Screen::SetFlags(Uint32 flags)
{
	// Saving flags value
	a_flags = flags;

	// Updating screen mode
	a_surface = SDL_SetVideoMode(a_rectangle->w, a_rectangle->h, 0, flags);
}

// Set screen resolution
void graphic::Screen::SetResolution(int w, int h)
{
	if(w != a_rectangle->w || h != a_rectangle->h)
	{
		a_surface = SDL_SetVideoMode(w, h, 0, a_flags);
		
		// Setting screen dimensions
		a_rectangle->w = w;
		a_rectangle->h = h;
	}
}

// Set fullscreen view mode
void graphic::Screen::FullScreen()
{
	if(!(a_flags & SDL_FULLSCREEN))
	{
		a_flags |= SDL_FULLSCREEN;
		
		// Updating screen mode
		a_surface = SDL_SetVideoMode(a_rectangle->w, a_rectangle->h, 0, a_flags);
	}
}

// Set window view mode
void graphic::Screen::Window()
{
	if(a_flags & SDL_FULLSCREEN)
	{
		a_flags &= ~SDL_FULLSCREEN;
		
		// Updating screen mode
		a_surface = SDL_SetVideoMode(a_rectangle->w, a_rectangle->h, 0, a_flags);
	}
}

// Save the whole screen into a backup surface
void graphic::Screen::Backup()
{
		// allocate the backup surface
		a_backup = SDL_CreateRGBSurface(a_flags,a_rectangle->w,a_rectangle->h,
										a_surface->format->BitsPerPixel,0,0,0,0); 
		
		// save the screen 
		SDL_BlitSurface(a_surface,NULL,a_backup,NULL);
}

// Restore the saved screen from a backup surface
void graphic::Screen::Restore()
{
		// restore the saved screen
		SDL_BlitSurface(a_backup,NULL,a_surface,NULL);

		// free the backup surface
		SDL_FreeSurface(a_backup);
}


// Update a given area of screen
void graphic::Screen::UpdateRect(SDL_Rect * a) 
{ 
	if(a->x < 0)
		a->x = 0;
	if(a->y < 0)
		a->y = 0;
	if(a->x + a->w > a_rectangle->w)
		a->x = a_rectangle->w - a->w;
	if(a->y + a->h > a_rectangle->h)
		a->y = a_rectangle->h - a->h;
	
	SDL_UpdateRect(a_surface, a->x, a->y, a->w, a->h); 
}
