#ifndef CLASS_SCREEN
#define CLASS_SCREEN

#include <SDL.h>
#include "BaseSurface.h"

namespace graphic
{
	
/// This is the class related to the surface representing the screen
class Screen : public BaseSurface
{
	private:
		/// backup surface used to backup/restore screen
		SDL_Surface * a_backup;
		/// screen flaqs
		Uint32 a_flags;

	public:
		/// Init screen surface
		Screen(int w, int h, Uint32 flags);
		/// destroyer
		~Screen() { };

		/// Set screen flags
		void SetFlags(Uint32 flags);
		/// Get setted screen flags
		Uint32 GetFlags() { return a_flags; };
		
		/// Set screen resolution
		void SetResolution(int w, int h);
		
		/// Set fullscreen view mode
		void FullScreen();
		/// Set window view mode
		void Window();
		
		/// Update the entire screen
		void Update() { SDL_Flip(a_surface); };
		/// Update a given area of screen
		void UpdateRect(SDL_Rect * a);

		/// Save the whole screen into a backup surface
		void Backup();
		/// Restore the saved screen from a backup surface
		void Restore();
		
		/// Get the Bits Per Pixel format (8 - 16 - 24 - 32)
		Uint8  GetBitPP() { return a_surface->format->BitsPerPixel; }; 
};

/// external pointer
extern Screen * screen;

}
#endif
