#include "SdlBlitter.h"

using namespace std;

// Blit an entire surface on screen (NULL, -> Rect)
void graphic::SdlBlitter::Blit(BaseSurface & src_surface)
{
	// source surface rect
	SDL_Rect * src_r = a_sf->GetRectangle(src_surface);
	// backup rect
	SDL_Rect * bkp_r = new SDL_Rect;
	
	// copy src surface rect values into backup rect
	bkp_r->x = src_r->x;
	bkp_r->y = src_r->y;
	bkp_r->w = src_r->w;
	bkp_r->h = src_r->h;
		
	// blit on screen
	SDL_BlitSurface(a_sf->GetSurface(src_surface), NULL, a_sf->GetSurface(*screen), bkp_r);
	
	delete bkp_r;
};

// Blit an entire surface on another one (NULL -> Rect)
void graphic::SdlBlitter::Blit(BaseSurface & src_surface, BaseSurface & dst_surface)
{
	// source surface rect
	SDL_Rect * src_r = a_sf->GetRectangle(src_surface);
	// backup rect
	SDL_Rect * bkp_r = new SDL_Rect;
	
	// copy src surface rect values into backup rect
	bkp_r->x = src_r->x;
	bkp_r->y = src_r->y;
	bkp_r->w = src_r->w;
	bkp_r->h = src_r->h;
		
	// blit on dst_surface
	SDL_BlitSurface(a_sf->GetSurface(src_surface), NULL, a_sf->GetSurface(dst_surface) , bkp_r);
	
	delete bkp_r;
}

		
// Blit part of a surface on screen (Rect -> NULL)
void graphic::SdlBlitter::Blit(BaseSurface & src_surface, SDL_Rect & src_rect)
{
	// backup rect
	SDL_Rect * bkp_r = new SDL_Rect;
	
	// copy src surface rect values into backup rect
	bkp_r->x = src_rect.x;
	bkp_r->y = src_rect.y;
	bkp_r->w = src_rect.w;
	bkp_r->h = src_rect.h;
		
	// blit on screen
	SDL_BlitSurface(a_sf->GetSurface(src_surface), bkp_r, a_sf->GetSurface(*screen), NULL);
	
	delete bkp_r;
}


// Blit part of a surface on another one (Rect -> NULL)
void graphic::SdlBlitter::Blit(BaseSurface & src_surface, SDL_Rect & src_rect, 
							   BaseSurface & dst_surface)
{
	// backup rect
	SDL_Rect * bkp_r = new SDL_Rect;
	
	// copy src surface rect values into backup rect
	bkp_r->x = src_rect.x;
	bkp_r->y = src_rect.y;
	bkp_r->w = src_rect.w;
	bkp_r->h = src_rect.h;
		
	// blit on dst_surface
	SDL_BlitSurface(a_sf->GetSurface(src_surface), bkp_r, a_sf->GetSurface(dst_surface), NULL);
	
	delete bkp_r;
}
		
// Blit part of a surface on part of screen (Rect -> Rect)
void graphic::SdlBlitter::Blit(BaseSurface & src_surface, SDL_Rect & src_rect, 
				  			   SDL_Rect & dst_rect)
{
	// backup rect
	SDL_Rect * bkp_r = new SDL_Rect;
	SDL_Rect * bkp2_r = new SDL_Rect;
	
	
	// copy src surface rect values into backup rect
	bkp_r->x = src_rect.x;
	bkp_r->y = src_rect.y;
	bkp_r->w = src_rect.w;
	bkp_r->h = src_rect.h;
	
		// copy src surface rect values into backup rect
	bkp2_r->x = dst_rect.x;
	bkp2_r->y = dst_rect.y;
	bkp2_r->w = dst_rect.w;
	bkp2_r->h = dst_rect.h;
		
	// blit on dst_surface
	SDL_BlitSurface(a_sf->GetSurface(src_surface), bkp_r, a_sf->GetSurface(*screen), bkp2_r);
	
	delete bkp_r;
	delete bkp2_r;
}

// Blit part of a surface on part of another one (Rect -> Rect)
void graphic::SdlBlitter::Blit(BaseSurface & src_surface, SDL_Rect & src_rect, 
				  			   BaseSurface & dst_surface, SDL_Rect & dst_rect)
{
	// backup rect
	SDL_Rect * bkp_r = new SDL_Rect;
	SDL_Rect * bkp2_r = new SDL_Rect;
	
	
	// copy src surface rect values into backup rect
	bkp_r->x = src_rect.x;
	bkp_r->y = src_rect.y;
	bkp_r->w = src_rect.w;
	bkp_r->h = src_rect.h;
	
		// copy src surface rect values into backup rect
	bkp2_r->x = dst_rect.x;
	bkp2_r->y = dst_rect.y;
	bkp2_r->w = dst_rect.w;
	bkp2_r->h = dst_rect.h;
		
	// blit on dst_surface
	SDL_BlitSurface(a_sf->GetSurface(src_surface), bkp_r, a_sf->GetSurface(dst_surface), bkp2_r);
	
	delete bkp_r;
	delete bkp2_r;	
}
