#include "Surface.h"
#include <iostream>

using namespace std;

// A new, not inizialized, surface 
graphic::Surface::Surface(Uint16 w, Uint16 h, bool trans, Uint32 flags, Uint16 bpp)
{ 
    Uint32 rmask, gmask, bmask, amask; 
    
    // transparent surface
    if(trans)
    {
        #if SDL_BYTEORDER == SDL_BIG_ENDIAN
        rmask = 0xff000000;
        gmask = 0x00ff0000;
        bmask = 0x0000ff00;
        amask = 0x000000ff;
        #else
        rmask = 0x000000ff;
        gmask = 0x0000ff00;
        bmask = 0x00ff0000;
        amask = 0xff000000;
        #endif
    }
    // opaque surface
    else
    {    
        rmask = 0;
        gmask = 0;
        bmask = 0;
        amask = 0; 
    }
    
	// create surface
    a_surface = SDL_CreateRGBSurface(flags, w, h, bpp, rmask, gmask, bmask, amask); 
    
    // set opacity to opaque (full visible)
    a_opacity =  SDL_ALPHA_OPAQUE;

	// Setting default position
	a_rectangle->x = 0;
	a_rectangle->y = 0;
	// Setting surface dimensions
	a_rectangle->w = w;
	a_rectangle->h = h;
};

// A new, not inizialized, surface 
graphic::Surface::Surface(Uint16 w, Uint16 h, SDL_PixelFormat * format, Uint32 flags)
{ 

    // create surface
    a_surface = SDL_CreateRGBSurface(flags|SDL_SRCALPHA,w,h, format->BitsPerPixel, 
                                     format->Rmask, format->Gmask, format->Bmask, format->Amask ); 
            
    // set opacity to opaque (full visible)
    a_opacity =  SDL_ALPHA_OPAQUE;

    // Setting default position
    a_rectangle->x = 0;
    a_rectangle->y = 0;
    // Setting surface dimensions
    a_rectangle->w = w;
    a_rectangle->h = h;
};

// Get the position of the upper left Point of the surface (x,y)
Point * graphic::Surface::GetPosition()
{
	Point *position = new Point;

	position->x = a_rectangle->x;
	position->y = a_rectangle->y;

	return position;
}

// Set opacity of the surface
void graphic::Surface::SetOpacity(Uint8 opacity)
{
	// save the opacity value
	a_opacity = opacity;
	// set the opacity
	SDL_SetAlpha(a_surface, SDL_SRCALPHA | SDL_RLEACCEL ,opacity);
}

// return true if the point (x,y) is inside the surface
bool graphic::Surface::IsInside(Sint16 x, Sint16 y)
{
    if(x >= a_rectangle->x && x <= (a_rectangle->x + a_rectangle->w) && 
       y >= a_rectangle->y && y <= (a_rectangle->y + a_rectangle->h))
        return true;
    else
        return false;    
}
