#ifndef CLASS_SURFACE
#define CLASS_SURFACE

#include "Screen.h"
#include "BaseSurface.h"

namespace graphic
{
	
/// The base graphic class, a generic surface
class Surface : public BaseSurface
{
	private:
		/// opacity value of the surface
		Uint8 a_opacity;

	public:
		/// Base constructor, just set the surface opaque (visible)
		Surface() { a_opacity =  SDL_ALPHA_OPAQUE; };
		/// A new, not inizialized, surface 
        Surface(Uint16 w, Uint16 h, bool trans = false, 
                Uint32 flags = SDL_SWSURFACE, Uint16 bpp = screen->GetBitPP()); 
        /// A new, not inizialized, surface 
        Surface(Uint16 w, Uint16 h, SDL_PixelFormat * format, Uint32 flags = SDL_SWSURFACE);
		/// Surface data is freed
		virtual ~Surface() { SDL_FreeSurface(a_surface); };

		/// Set position of the upper left Point of the surface (x,y)
		/// passing a Point
		virtual void SetPosition(Point position)
		{
			a_rectangle->x = position.x;
			a_rectangle->y = position.y;
		};
		/// Set position of the upper left Point of the surface (x,y) 
		/// passing x and y values
		virtual void SetPosition(Sint16 x, Sint16 y)
		{
			a_rectangle->x = x;
			a_rectangle->y = y;
		};
		/// Set X position of the upper left Point of the surface 
		virtual void SetXPosition(Sint16 x)	{ a_rectangle->x = x; };
		/// Set Y position of the upper left Point of the surface 
		virtual void SetYPosition(Sint16 y)	{ a_rectangle->y = y; };
		
		/// Get the position of the upper left Point of the surface (x,y)
		Point * GetPosition();

		/// Set opacity of the surface
		void SetOpacity(Uint8 opacity);
		/// Get opacity attribute
		Uint8 GetOpacity() { return a_opacity; };
  
        /// return true if the point p is inside the surface
        bool IsInside(Point * p) { return IsInside(p->x, p->y); };
        /// return true if the point (x,y) is inside the surface
        bool IsInside(Sint16 x, Sint16 y);
};

}

#endif
