/* 	
 *  	
 *  	Mars, Land Of No Mercy - 0.1.1
 *
 *	
 *	This is a release that fixs all the bugs and improve several 
 *	features of the 0.1.0.
 *
 *	
 *	features of this release are:
 *	
 *	- a better scrolling
 *	- a better selection highlighter
 *	- 2 tilesets updated
 *	- SWA mechs updated
 *	- several bug fixes
 *	
 *
 * 	------------------------------------------------------------
 
 * 	All the source code is released under GPL licence, you can 
 * 	find a copy of this licence at:
 * 	http://www.gnu.org/copyleft/gpl.html
 *
 * 	------------------------------------------------------------
 * 	
 *  Links:
 *  
 * 	homepage: http://mars.sf.net
 * 	forum: http://mars.dev-labs.net/forum/
 * 	irc: irc.freenode.org - #nomercy
 *
 */

#include <string>
#include <vector>

#include <SDL.h>

#include "graphic/AlphaImage.h"
#include "graphic/Image.h"
#include "graphic/MovableAlphaImage.h"
#include "graphic/MovableImage.h"
#include "graphic/MovableSurface.h"
#include "graphic/Screen.h"
#include "graphic/SdlBlitter.h"
#include "graphic/Surface.h"

#include "Building.h"
#include "DataTypes.h"
#include "Functions.h"
#include "IsoFow.h"
#include "IsoGrid.h"
#include "IsoMiniMap.h"
#include "IsoMouse.h"
#include "IsoObject.h"
#include "IsoObjectsMap.h"
#include "IsoTilesMap.h"
#include "MiniMapPanel.h"
#include "Path.h"
#include "Pathfind.h"
#include "PathHighlighter.h"
#include "PathNode.h"
#include "SceneRenderer.h"
#include "SelectionHighlighter.h"
#include "SelectionData.h"
#include "StatisticalValue.h"
#include "SceneElement.h"
#include "Unit.h"

// minimum offset betwenn screen and widgets
#define SW_OFFSET 5

// map dimensions
#define ROWS 50
#define COLS 50

// frame control
#define DEFAULT_FPS 100
#define MS_X_SEC 1000
#define FRAME_TIME MS_X_SEC / DEFAULT_FPS

// transparency value of the mmap
#define TRANS 100

#define ROTATION_COST 2

#define OBJ_XMOV 4
#define OBJ_YMOV OBJ_XMOV / 2

using namespace std;
using namespace graphic;

Screen * graphic::screen;
Blitter * graphic::blitter;

int main(int argc, char **argv)
{
	// the system video is initialized
    SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER);
	// init SDL_ttf
	TTF_Init();

	// when exit, execute SDL_Quit to restore everything
	atexit(SDL_Quit);
	
	Dimension screen_dim = { RIS2_W, RIS2_H };	
	
	// init screen
	screen = new Screen(screen_dim.w, screen_dim.h, SDL_SWSURFACE|SDL_FULLSCREEN);    
	
	// a new blitter
	blitter = new SdlBlitter;
	
	// options (resolution & tileset)
	Options opt;
	
	// default res is 1024 x 768
	opt.ris.w = RIS2_W;
	opt.ris.h = RIS2_H;
	
	// start menu
	menu(opt);
	
	// exit
	if(!(opt.choice))
		return 0;
	
	// change resolution
	if(opt.ris.w != screen_dim.w)
	{
		screen_dim.w = opt.ris.w;
		screen_dim.h = opt.ris.h;
		
		screen->SetResolution(screen_dim.w, screen_dim.h);
	}

	SDL_Color trans = {0xFF, 0, 0xFF, 0};

    // select tileset to load
	string dir;		
	if(opt.choice == 1)
		dir = "data/img/tiles/128x64/terrain_01/";
	else if(opt.choice == 2)
		dir = "data/img/tiles/128x64/terrain_02/";
    else if(opt.choice == 3)
        dir = "data/img/tiles/128x64/terrain_03/";
	
	// minimap tiles
	string dir_mm = "data/img/tiles/minimap/";

    /* --- ELEMENTS --- */
    AlphaImage * aimg;  
    StatisticalValue * stat;
    
    /* SCENE ELEMENT 1 */
    vector < Image * > views_se1;  
    aimg = new AlphaImage("data/img/scene_elements/mech_ruins_1/01.png"); 
    views_se1.push_back(aimg);
    aimg = new AlphaImage("data/img/scene_elements/mech_ruins_1/01.png");
    views_se1.push_back(aimg);
        
    vector < StatisticalValue * > stats_se1;
    stat = new StatisticalValue(300);
    stats_se1.push_back(stat);
    
    SceneElement element1(1, 1, views_se1, stats_se1);
    
    /* SCENE ELEMENT 2 */
    vector < Image * > views_se2;    
    aimg = new AlphaImage("data/img/scene_elements/fuel_drums_1/01.png");
    views_se2.push_back(aimg);
    aimg = new AlphaImage("data/img/scene_elements/fuel_drums_1/02.png");
    views_se2.push_back(aimg);
            
    vector < StatisticalValue * > stats_se2;
    stat = new StatisticalValue(200);
    stats_se2.push_back(stat);
    
    SceneElement element2(1, 1, views_se2, stats_se2);
    
    /* SCENE ELEMENT 3 */
    vector < Image * > views_se3;    
    aimg = new AlphaImage("data/img/buildings/NGG/bunker_outpost_destr_1/01.png");
    views_se3.push_back(aimg);
    aimg = new AlphaImage("data/img/buildings/NGG/bunker_outpost_destr_1/02.png");
    views_se3.push_back(aimg);
            
    vector < StatisticalValue * > stats_se3;
    stat = new StatisticalValue(2000);
    stats_se3.push_back(stat);
    
    SceneElement element3(3, 3, views_se3, stats_se3);
    
    /* SCENE ELEMENT 4 */           
    vector < StatisticalValue * > stats_se4;
    stat = new StatisticalValue(200);
    stats_se4.push_back(stat);
    
    SceneElement element4(1, 1, views_se2, stats_se4);
    
    /* SCENE ELEMENT 5 */           
    vector < StatisticalValue * > stats_se5;
    stat = new StatisticalValue(200);
    stats_se5.push_back(stat);
    
    SceneElement element5(1, 1, views_se2, stats_se5);
    
    /* BUILDING 1 */    
    vector < Image * > views_b1;
    aimg = new AlphaImage("data/img/buildings/NGG/bunker_outpost_1/01.png");
    views_b1.push_back(aimg);
    aimg = new AlphaImage("data/img/buildings/NGG/bunker_outpost_1/02.png");
    views_b1.push_back(aimg);  
    views_b1.push_back(aimg);

    vector < StatisticalValue * > stats_b1;    
    // ENERGY
    stat = new StatisticalValue(1000);
    stats_b1.push_back(stat);
    // VISIBILITY RANGE
    stat = new StatisticalValue(2);
    stats_b1.push_back(stat);              
    
    Building build1(3, 3, views_b1, stats_b1);
    build1.SetFaction(F_NGG);    
    
    /* BUILDING 2 */    
    vector < Image * > views_b2;
    aimg = new AlphaImage("data/img/buildings/SWA/fortress_1/01.png");
    views_b2.push_back(aimg);
    aimg = new AlphaImage("data/img/buildings/SWA/fortress_1/02.png");
    views_b2.push_back(aimg);  
    views_b2.push_back(aimg);

    vector < StatisticalValue * > stats_b2;    
    // ENERGY
    stat = new StatisticalValue(2000);
    stats_b2.push_back(stat);
    // VISIBILITY RANGE
    stat = new StatisticalValue(2);
    stats_b2.push_back(stat);              
    
    Building build2(3, 3, views_b2, stats_b2);
    build2.SetFaction(F_SWA);    
    
    /* BUILDING 3 */    
    vector < Image * > views_b3;
    aimg = new AlphaImage("data/img/buildings/SWA/cannons_1/01.png");
    views_b3.push_back(aimg);
    aimg = new AlphaImage("data/img/buildings/SWA/cannons_1/02.png");
    views_b3.push_back(aimg);  
    views_b3.push_back(aimg);

    vector < StatisticalValue * > stats_b3;    
    // ENERGY
    stat = new StatisticalValue(500);
    stats_b3.push_back(stat);
    // VISIBILITY RANGE
    stat = new StatisticalValue(2);
    stats_b3.push_back(stat);              
    
    Building build3(2, 2, views_b3, stats_b3);
    build3.SetFaction(F_SWA);   
    
    /* UNIT 1 - HEAVY MECH NGG */
    vector < Image * > views_u1;
    aimg = new AlphaImage("data/img/units/NGG/mech_heavy_1/01.png");
    views_u1.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_heavy_1/02.png");
    views_u1.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_heavy_1/03.png");
    views_u1.push_back(aimg);   
    aimg = new AlphaImage("data/img/units/NGG/mech_heavy_1/04.png");
    views_u1.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_heavy_1/05.png");
    views_u1.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_heavy_1/06.png");
    views_u1.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_heavy_1/07.png");
    views_u1.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_heavy_1/08.png");
    views_u1.push_back(aimg);
    
    vector < StatisticalValue * > stats_u1;    
    // ENERGY
    stat = new StatisticalValue(300);
    stats_u1.push_back(stat);
    // VISIBILITY RANGE
    stat = new StatisticalValue(5);
    stats_u1.push_back(stat);
    // ACTION POINTS
    stat = new StatisticalValue(60);
    stats_u1.push_back(stat);
    Unit unit1(1, 1, views_u1, stats_u1);
    unit1.SetFaction(F_NGG);
    unit1.SetDirection(NORTH);
    
    /* UNIT 2  - MEDIUM MECH NGG */
    vector < Image * > views_u2;
    aimg = new AlphaImage("data/img/units/NGG/mech_medium_1/01.png");
    views_u2.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_medium_1/02.png");
    views_u2.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_medium_1/03.png");
    views_u2.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_medium_1/04.png");
    views_u2.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_medium_1/05.png");
    views_u2.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_medium_1/06.png");
    views_u2.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_medium_1/07.png");
    views_u2.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_medium_1/08.png");
    views_u2.push_back(aimg);

    vector < StatisticalValue * > stats_u2;    
    // ENERGY
    stat = new StatisticalValue(200);
    stats_u2.push_back(stat);
    // VISIBILITY RANGE
    stat = new StatisticalValue(6);
    stats_u2.push_back(stat);
    // ACTION POINTS
    stat = new StatisticalValue(60);
    stats_u2.push_back(stat);
    
    Unit unit2(1, 1, views_u2, stats_u2);
    unit2.SetFaction(F_NGG);
    unit2.SetDirection(NORTH);
    
    /* UNIT 3 - LIGHT MECH NGG */
    vector < Image * > views_u3;
    aimg = new AlphaImage("data/img/units/NGG/mech_light_1/01.png");
    views_u3.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_light_1/02.png");
    views_u3.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_light_1/03.png");
    views_u3.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_light_1/04.png");
    views_u3.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_light_1/05.png");
    views_u3.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_light_1/06.png");
    views_u3.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_light_1/07.png");
    views_u3.push_back(aimg);
    aimg = new AlphaImage("data/img/units/NGG/mech_light_1/08.png");
    views_u3.push_back(aimg);
    
    vector < StatisticalValue * > stats_u3;    
    // ENERGY
    stat = new StatisticalValue(100);
    stats_u3.push_back(stat);
    // VISIBILITY RANGE
    stat = new StatisticalValue(6);
    stats_u3.push_back(stat);
    // ACTION POINTS
    stat = new StatisticalValue(70);
    stats_u3.push_back(stat);
    
    Unit unit3(1, 1, views_u3, stats_u3);
    unit3.SetFaction(F_NGG);
    unit3.SetDirection(NORTH);
    
    /* UNIT 4 - HEAVY MECH SWA */
    vector < Image * > views_u4;
    aimg = new AlphaImage("data/img/units/SWA/mech_heavy_1/01.png");
    views_u4.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_heavy_1/02.png");
    views_u4.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_heavy_1/03.png");
    views_u4.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_heavy_1/04.png");
    views_u4.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_heavy_1/05.png");
    views_u4.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_heavy_1/06.png");
    views_u4.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_heavy_1/07.png");
    views_u4.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_heavy_1/08.png");
    views_u4.push_back(aimg);
    
    vector < StatisticalValue * > stats_u4;    
    // ENERGY
    stat = new StatisticalValue(300);
    stats_u4.push_back(stat);
    // VISIBILITY RANGE
    stat = new StatisticalValue(5);
    stats_u4.push_back(stat);
    // ACTION POINTS
    stat = new StatisticalValue(60);
    stats_u4.push_back(stat);
    
    Unit unit4(1, 1, views_u4, stats_u4);
    unit4.SetFaction(F_SWA);
    unit4.SetDirection(NORTH_EAST);
    
    /* UNIT 5  - MEDIUM MECH SWA */
    vector < Image * > views_u5;
    aimg = new AlphaImage("data/img/units/SWA/mech_medium_1/01.png");
    views_u5.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_medium_1/02.png");
    views_u5.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_medium_1/03.png");
    views_u5.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_medium_1/04.png");
    views_u5.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_medium_1/05.png");
    views_u5.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_medium_1/06.png");
    views_u5.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_medium_1/07.png");
    views_u5.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_medium_1/08.png");
    views_u5.push_back(aimg);

    vector < StatisticalValue * > stats_u5;    
    // ENERGY
    stat = new StatisticalValue(200);
    stats_u5.push_back(stat);
    // VISIBILITY RANGE
    stat = new StatisticalValue(6);
    stats_u5.push_back(stat);
    // ACTION POINTS
    stat = new StatisticalValue(60);
    stats_u5.push_back(stat);
    
    Unit unit5(1, 1, views_u5, stats_u5);
    unit5.SetFaction(F_SWA);
    unit5.SetDirection(EAST);
    
    /* UNIT 6 - LIGHT MECH SWA */
    vector < Image * > views_u6;
    aimg = new AlphaImage("data/img/units/SWA/mech_light_1/01.png");
    views_u6.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_light_1/02.png");
    views_u6.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_light_1/03.png");
    views_u6.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_light_1/04.png");
    views_u6.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_light_1/05.png");
    views_u6.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_light_1/06.png");
    views_u6.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_light_1/07.png");
    views_u6.push_back(aimg);
    aimg = new AlphaImage("data/img/units/SWA/mech_light_1/08.png");
    views_u6.push_back(aimg);
    
    vector < StatisticalValue * > stats_u6;    
    // ENERGY
    stat = new StatisticalValue(100);
    stats_u6.push_back(stat);
    // VISIBILITY RANGE
    stat = new StatisticalValue(6);
    stats_u6.push_back(stat);
    // ACTION POINTS
    stat = new StatisticalValue(70);
    stats_u6.push_back(stat);
    
    Unit unit6(1, 1, views_u6, stats_u6);
    unit6.SetFaction(F_SWA);
    unit6.SetDirection(NORTH);
    
    /* --- END ELEMENTS --- */
    
	// load the tileset of the map
	TileSet ts(dir, &trans);
 
	// load the tileset of the minimap
	TileSet ts_mm(dir_mm, &trans);

	// random generated tiles map
	IsoTilesMap tm(ROWS, COLS, &ts);
	
	// temporary objects map
    IsoObjectsMap om(&tm);
        
    // NGG
    om.AddObject(&element1, 10, 14);
    om.AddObject(&element2, 9, 4);
    om.AddObject(&element3, 20, 9);    
    om.AddObject(&build1, 5, 4);        
    om.AddObject(&unit1, 12, 7);
    om.AddObject(&unit2, 25, 25);
    om.AddObject(&unit3, 6, 20);
    
    // SWA
    om.AddObject(&element4, 38, 13);
    om.AddObject(&element5, 43, 10);
    om.AddObject(&build2, 40, 5); 
    om.AddObject(&build3, 36, 20);  
    om.AddObject(&unit4, 35, 12);
    om.AddObject(&unit5, 29, 19);
    om.AddObject(&unit6, 41, 16);
    
    build1.SubCurStat(STAT_ENERGY, 250);
    build2.SubCurStat(STAT_ENERGY, 1000);
    unit3.SubCurStat(STAT_ENERGY, 10);
    unit6.SubCurStat(STAT_ENERGY, 60);
    
    // building visibility
    om.ComputeObjVisibility(&build1, build1.GetCurStat(STAT_VISIBILITY));
    // unit visibility
    om.ComputeObjVisibility(&unit1, unit1.GetCurStat(STAT_VISIBILITY), unit1.GetDirection());
    // unit visibility
    om.ComputeObjVisibility(&unit2, unit2.GetCurStat(STAT_VISIBILITY), unit2.GetDirection());
    // unit visibility
    om.ComputeObjVisibility(&unit3, unit3.GetCurStat(STAT_VISIBILITY), unit3.GetDirection());    

	// iso Fow
    Image * black_tile = new Image("data/img/tiles/128x64/black.png", trans);
    IsoFow fow(&tm, black_tile);
    
    // iso Grid
    Image * black_grid = new Image("data/img/grids/128x64/black_grid.png", trans);
    Image * white_grid = new Image("data/img/grids/128x64/white_grid.png", trans);    
    Image * white_grid2 = new Image("data/img/grids/128x64/white_grid.png", trans);    
    IsoGrid grid(&tm, black_grid, white_grid, white_grid2);
    
	// iso Minimap
	IsoMiniMap mm(&tm, &om, &ts_mm);
  
    /* SELECTION HIGHLIGHTER */
    Image * img;
    vector< Image * > high_images;    
    img = new Image("data/img/highlighter/01.png", trans);
    high_images.push_back(img);
    img = new Image("data/img/highlighter/02.png", trans);
    high_images.push_back(img);
    img = new Image("data/img/highlighter/03.png", trans);
    high_images.push_back(img);
    img = new Image("data/img/highlighter/04.png", trans);    
    high_images.push_back(img);
    
    SelectionHighlighter sel(high_images);
                
    /* SELECTION DATA */
    
    vector< Image * > sdata_images;    
    img = new Image("data/img/sel_data/ebar_border.png", trans);
    sdata_images.push_back(img);
    img = new Image("data/img/sel_data/ebar_good.png");
    sdata_images.push_back(img);
    img = new Image("data/img/sel_data/ebar_bad.png");
    sdata_images.push_back(img);
        
    SelectionData sel_data(sdata_images);
       
    /* PATH HIGHLIGHTER */
    Image * high_tile_green = new Image("data/img/tiles/128x64/green.png", trans);
    Image * high_tile_red = new Image("data/img/tiles/128x64/red.png", trans);
    Image * high_tile_yellow = new Image("data/img/tiles/128x64/yellow.png", trans);
    PathHighlighter path_h(&tm, high_tile_green, high_tile_red, high_tile_yellow);    
    /* PATH HIGHLIGHTER END */
         
    Pathfind path_finder(&tm);
    Path * gen_path = NULL;            

	// current view (screen position on the map) rect 
	SDL_Rect view = { ((tm.GetW() - screen_dim.w) / 2), 0, screen_dim.w, screen_dim.h };
    
	// upper limit of view
	Sint16 u_limit = -( view.h / 2);
	// bottom limit of view
	Sint16 b_limit = tm.GetH() - ( view.h / 2);
	// left limit of view
	Sint16 l_limit = -( view.w / 2);
	// right limit of view
	Sint16 r_limit = tm.GetW() - ( view.w / 2);
	
    /* WINDOW MINIMAP */
 
	// minimap window border
    MovableImage mm_border_1024("data/img/mmap_border_1024.png", trans);
    MovableImage mm_border_800("data/img/mmap_border_800.png", trans);
	// minimap view rect
    Image mm_view_1024("data/img/mmap_view_1024.png", trans);
    Image mm_view_800("data/img/mmap_view_800.png", trans);
	// surface used by the minimap
	Surface mm_area_1024(mm_border_1024.GetW() - 2, mm_border_1024.GetH() - 2);
	Surface mm_area_800(mm_border_800.GetW() - 2, mm_border_800.GetH() - 2);
	
	// window surface
	MovableSurface mm_window_1024(mm_area_1024.GetW(), mm_area_1024.GetH());
	MovableSurface mm_window_800(mm_area_800.GetW(), mm_area_800.GetH());
	
	// pointers 
	Image * mm_view;
	MovableImage * mm_border;
	MovableSurface * mm_window;
	
	// default data are for 1024x768	
	mm_border = & mm_border_1024;
	mm_view = & mm_view_1024;
	mm_window = & mm_window_1024;
	
	// default resolution changed to 800x600
	if(screen_dim.w == RIS1_W)
	{
		mm_border = & mm_border_800;
		mm_view = & mm_view_800;
		mm_window = & mm_window_800;
	}
	
	// set position of the minimap border, bottom-right corner
	mm_border->SetInitialPosition(screen_dim.w - mm_border->GetW(), 
						   screen_dim.h - mm_border->GetH());
				
	// set position of minimap window
	mm_window->SetInitialPosition(screen_dim.w - mm_window->GetW() - 1, 
						 screen_dim.h - mm_window->GetH() - 1);
		
	// visible limits of the minimap
	SDL_Rect mm_limit = { (mm.GetW() - mm_window->GetW()) / 2, 
						  0, mm_window->GetW() , mm_window->GetH()};
    SDL_Rect mm_limit_800 = { (mm.GetW() - mm_window_800.GetW()) / 2, 
                              0, mm_window_800.GetW() , mm_window_800.GetH()};

	// upper-left point of view rect into the minimap window
	Point mm_view_p = { view.x / mm.GetScale() - mm_limit.x, 
						view.y / mm.GetScale() - mm_limit.y}; 
	
	// set position of view rect into the minimap window
	mm_view->SetPosition(mm_view_p);
   
    /* END WINDOW MINIMAP */    
 
    /* MINIMAP PANEL */
    MiniMapPanel * mm_panel;    
    MiniMapPanel * mm_panel_1024 = new MiniMapPanel(&mm, &mm_view_1024, mm_limit);    
    MiniMapPanel * mm_panel_800 = new MiniMapPanel(&mm, &mm_view_800, mm_limit_800);
    
    if(screen_dim.w == RIS2_W)
        mm_panel = mm_panel_1024;
    else
        mm_panel = mm_panel_800;
    
    mm_panel->SetRendererLimits(u_limit, b_limit, l_limit, r_limit);

    // isometric mouse
    MovableAlphaImage * maimg;
    vector < MovableImage * > pointers;
    maimg = new MovableAlphaImage("data/img/pointer.png");
    pointers.push_back(maimg);
    maimg = new MovableAlphaImage("data/img/pointer2.png");
    pointers.push_back(maimg);
    IsoMouse * mouse = new IsoMouse(&tm, pointers); 
    mouse->SetView(view.x, view.y);    
    
    // turn the system cursor off
    SDL_ShowCursor(0);
 
    /* SCENE RENDERER INIT*/
    SceneRenderer renderer(view);
    renderer.SetLimits(u_limit, b_limit, l_limit, r_limit);
	    
    tm.SetBuffering();
    renderer.AddLayer(&tm);
        
    fow.SetBuffering();
    renderer.AddLayer(&fow);
    
    grid.SetBuffering();
    renderer.AddLayer(&grid);
    
    path_h.SetBlit(false);
    renderer.AddLayer(&path_h);
    
    renderer.AddLayer(&om);
    
    sel.SetBlit(false);
    renderer.AddLayer(&sel);
    
    sel_data.SetBlit(false);
    renderer.AddLayer(&sel_data);
    /* END - SCENE RENDERER INIT*/
                    
    // refresh the buffer;
    renderer.RefreshBuffer();
    // blit the scene
    renderer.BufferedBlitScene(*screen);        
    
    /* WINDOW */
	// save background (screen) of minimap window
	mm_border->SaveBackground(screen);

    mm_panel->Blit(mm_window);
    
	// blit minimap window border
	blitter->Blit(*mm_border);	
	// by default blit the window as transparent
	mm_window->SetOpacity(TRANS);
	blitter->Blit(*mm_window);
    /* END WINDOW */
        
    /* ROTATION */
    FloatPoint * f_origin;
 
    mouse->Blit();        

	// update the screen
	screen->Update();

	// game loop condition flag
	bool done = false;
	
	//SDL_Event struct used to manage events
	SDL_Event event;
	
	// mouse position
	Point * prev_mouse_pos = new Point;
	
	// flag that says if mouse has been clicked
	bool mouse_clicked = false;
	// flag that says if mouse is dragging
	bool dragging = false;
	// flag that says if the grid has to be blitted or not
	bool grid_on = true;
	// flag that says if the minimap window is opaque/ON or transparent/OFF
	bool mmap_on = false;
 
    // unit movement destination
    Sint16 x_dest = 0;
    Sint16 y_dest = 0;
        
	// drag movement of the mouse
	int drag_mov_x, drag_mov_y;
 
    // pointers used by selection
    Element * temp_elem = NULL;  
    FactionElement * sel_elem = NULL;  
    Unit * sel_unit = NULL;
    Building * sel_build = NULL;
    
    bool elem_selected = false;

    CellInd * temp_cell = NULL;    
    
    int orientation;
   
    bool unit_moving = false;
    
    bool checkp_reached = false;

    PathNode * cur_pnode = NULL;

    Uint32 numframes = 0;
	
	// times used by frame control
	int t_delay, t_diff; 
	Uint32 t1, t2; 
	// times used by pixel movement control
	Uint32 t1_mov, t2_mov;	
	// init times
	t1 = t1_mov = SDL_GetTicks(); 

	//game loop 
	while(!done)	
	{
        numframes++;
             
		while(SDL_PollEvent(&event))
		{
			// the program quits if someone press the x simbol of the window with the mouse 
			if( event.type == SDL_QUIT )  
				done = true;  
			
			// key pressed
			else if( event.type == SDL_KEYDOWN )
			{
				//quit if someone press ESC 
				switch(event.key.keysym.sym)
				{       
					// fullscreen
					case SDLK_f:
						// save screen
						screen->Backup();
						// go in fullscreen mode
						screen->FullScreen();
						//restore screen
						screen->Restore();
						screen->Update();
						break;
					
					// window mode
					case SDLK_w:
						// save window 
						screen->Backup();
						// go in window mode
						screen->Window();
						// restore window
						screen->Restore();
						screen->Update();
						break;
						
					// ESC -> exit
					case SDLK_ESCAPE:
						done = true; 
						break;
						
					// 800x600
					case SDLK_F1:
						if(screen_dim.w != RIS1_W)
						{
							// update screen dimension
							screen_dim.w = RIS1_W;
							screen_dim.h = RIS1_H;
                            screen->SetResolution(RIS1_W, RIS1_H);
                            
                            u_limit = -( RIS1_H / 2);
                            b_limit = tm.GetH() - ( RIS1_H / 2);
                            l_limit = -( RIS1_W / 2);
                            r_limit = tm.GetW() - ( RIS1_W / 2);
                            
                            renderer.SetLimits(u_limit, b_limit, l_limit, r_limit);                            
						
							// update scene view rect
                            renderer.UpdateViewDimensions(RIS1_W, RIS1_H);
                           
							// update dimension-related data
							mm_border = & mm_border_800;
							mm_window = & mm_window_800;
							
							// set position of minimap window border
							mm_border->SetInitialPosition(screen_dim.w - mm_border->GetW(), 
												   screen_dim.h - mm_border->GetH());
							
							// update visible rect of the miniamp
							mm_limit.w = mm_window->GetW();
							mm_limit.h = mm_window->GetH();
		
							// set minimap window position
							mm_window->SetInitialPosition(screen_dim.w - mm_window->GetW() - 1, 
												 screen_dim.h - mm_window->GetH() - 1);

                            mm_panel = mm_panel_800;    
                            mm_panel->SetRendererLimits(u_limit, b_limit, l_limit, r_limit);
                            mm_panel->UpdateView(renderer.GetViewX(), renderer.GetViewY());  
                            
                            mmap_on = false;
                            mm_window->SetOpacity(TRANS);
                            
							// refresh the scene							
                            renderer.RequestBufferRefresh();
                            renderer.RequestBlit(BUFFERED);
						}
						break; 
						
					case SDLK_F2:
						if(screen_dim.w != RIS2_W)
						{
							// update screen dimension
							screen_dim.w = RIS2_W;
							screen_dim.h = RIS2_H;
                            screen->SetResolution(RIS2_W, RIS2_H);
                            
                            u_limit = -( RIS2_H / 2);
                            b_limit = tm.GetH() - ( RIS2_H / 2);
                            l_limit = -( RIS2_W / 2);
                            r_limit = tm.GetW() - ( RIS2_W / 2);
                            
                            renderer.SetLimits(u_limit, b_limit, l_limit, r_limit); 
                        
                            // update scene view rect
                            renderer.UpdateViewDimensions(RIS2_W, RIS2_H);
 
							// update dimension-related data
							mm_border = & mm_border_1024;
							mm_window = & mm_window_1024;
									
							// set position of minimap window border	
                            mm_border->SetInitialPosition(screen_dim.w - mm_border->GetW(), 
												   screen_dim.h - mm_border->GetH());
	
							// update visible rect of the miniamp
							mm_limit.w = mm_window->GetW();
							mm_limit.h = mm_window->GetH();

							// set minimap window position
							mm_window->SetPosition(screen_dim.w - mm_window->GetW() - 1, 
												 screen_dim.h - mm_window->GetH() - 1);
      
                            mm_panel = mm_panel_1024;
                            mm_panel->UpdateView(renderer.GetViewX(), renderer.GetViewY());
                            
                            mmap_on = false;
                            mm_window->SetOpacity(TRANS);
       
							// refresh the scene
                            renderer.RequestBufferRefresh();
                            renderer.RequestBlit(BUFFERED);
						}
						break;
      
                        // restore Unit Action Points
                        case SDLK_SPACE:
                            if(!unit_moving && sel_unit != NULL)
                            {
                                sel_unit->RestoreCurStat(STAT_ACTION);
                                
                                // recompute the path to update the remaining AP
                                if(mouse->IsInsideMap() && 
                                   (tm.IsWalkable(mouse->GetRow(), mouse->GetCol()) || 
                                    tm.IsVisible(mouse->GetRow(), mouse->GetCol()) != VISIBLE) )
                                {
                                    if(gen_path != NULL)
                                        delete gen_path;
                                                
                                    gen_path = path_finder.MakePath(sel_unit->GetRow(), sel_unit->GetCol(), 
                                            mouse->GetRow(), mouse->GetCol(), 
                                            sel_unit->GetDirection());
                        
                                    path_h.SetPath(gen_path);
                                                 
                                    path_h.SetRemainingAP(sel_unit->GetCurStat(STAT_ACTION) - gen_path->GetTotalCost());
                                    
                                    path_h.SetBlit();
                                }
                                                        
                                // refresh                                
                                renderer.RequestBlit(BUFFERED);
                            }
                            break;
                            
                        // change mouse pointer
                        case SDLK_1:
                            mouse->SetPointer(0);
                            renderer.RequestBlit(BUFFERED);
                            break;          
                        // change mouse pointer
                        case SDLK_2:
                            mouse->SetPointer(1);
                            renderer.RequestBlit(BUFFERED);
                            break;              
                                      
					default:
						break;
				}
			}
			
			// key released
			else if( event.type == SDL_KEYUP )
			{                
				switch(event.key.keysym.sym)
				{      
                    // G -> switch grid on/off
                    case SDLK_g:
                        if(grid_on)
                            grid_on = false;
                        else
                            grid_on = true;
                        
                        grid.SetBlit(grid_on); 
                                                                                                  
                        renderer.RequestBufferRefresh();
                        renderer.RequestBlit(BUFFERED);
                        
                        break;
						
					default:
						break;
				}     

			}
           
			// moving the mouse
			else if( event.motion.type == SDL_MOUSEMOTION)
			{
                // store current mouse position
                mouse->SetPosition(event.button.x, event.button.y, renderer.GetViewX(), renderer.GetViewY());
                
                // check for scrolling
                renderer.MouseScrolling(mouse->GetX(), mouse->GetY());

                // cell changed, unit selected and not moving
                if(mouse->IsCellChanged() && !unit_moving && sel_unit && sel_unit->GetFaction() == F_NGG)
                {                    
                    // the mouse is inside the map and the 
                    // current cell is walkable or not visible
                    if(mouse->IsInsideMap() && 
                       (tm.IsWalkable(mouse->GetRow(), mouse->GetCol()) || 
                        tm.IsVisible(mouse->GetRow(), mouse->GetCol()) != VISIBLE) )
                    {
                        // a path has just been made
                        if(gen_path != NULL)
                            delete gen_path;
                                                
                        // made a new path
                        gen_path = path_finder.MakePath(sel_unit->GetRow(), sel_unit->GetCol(), 
                                                        mouse->GetRow(), mouse->GetCol(), 
                                                        sel_unit->GetDirection());  
                        
                        // set the path highlighter and ask for blit
                        path_h.SetPath(gen_path);
                        path_h.SetRemainingAP(sel_unit->GetCurStat(STAT_ACTION) - gen_path->GetTotalCost());
                                                  
                        path_h.SetBlit();
                        
                        // buffered render
                        renderer.RequestBlit(BUFFERED);                       
                    }
                    // mouse is outside the map or on a not walkable cell => delete the current path
                    else if(gen_path != NULL)
                    {
                        // delete path
                        delete gen_path;
                        gen_path = NULL;
                                                
                        // not ask for path blit
                        path_h.SetBlit(false);
                        
                        // buffered render
                        renderer.RequestBlit(BUFFERED);
                    }
                }
                
				// mouse is clicked + motion => dragging
				if(mouse_clicked)
				{
					dragging = true;
                        
                    /* IMPORTANT */
                    // restore the screen under the mouse pointer so to remove the mouse
                    // image before moving the window
                    mouse->RestoreBackg();
					
					// restore background (screen) under the window
					mm_border->RestoreBackground(screen);

					// compute the movement
                    drag_mov_x = mouse->GetX() - prev_mouse_pos->x;
                    drag_mov_y = mouse->GetY() - prev_mouse_pos->y;
                    
					// check for boundary exception
					if((mm_border->GetX() - drag_mov_x) <= SW_OFFSET && drag_mov_x < 0)
						drag_mov_x = -mm_border->GetX();
					if((mm_border->GetX() + mm_border->GetW() + drag_mov_x) >= screen_dim.w - SW_OFFSET && 
					   drag_mov_x > 0)
						drag_mov_x = screen_dim.w - (mm_border->GetX() + mm_border->GetW());
					if((mm_border->GetY() - drag_mov_y) <= SW_OFFSET && drag_mov_y < 0)
						drag_mov_y = -mm_border->GetY();
					if((mm_border->GetY() + mm_border->GetH() + drag_mov_y) >= screen_dim.h - SW_OFFSET && 
					   drag_mov_y > 0)
						drag_mov_y = screen_dim.h - (mm_border->GetY() + mm_border->GetH());
					
					// move border and the window area
					mm_border->Move(drag_mov_x, drag_mov_y);	
					mm_window->Move(drag_mov_x, drag_mov_y);

					// save new background (screen)
					mm_border->SaveBackground(screen);	
					// blit the window border
					blitter->Blit(*mm_border);
					// blit the window surface
					blitter->Blit(*mm_window);					
					// update altered part of screen
					mm_border->UpdateScreen();

					// store previous mouse position
                    prev_mouse_pos->x = mouse->GetX();
                    prev_mouse_pos->y = mouse->GetY();
     
                    // move the mouse and blit
                    mouse->MoveAndBlit();
				}            	
                else				
                    // restore the screen and blit
                    mouse->UpdateAndBlit();
			}
			
			// left mouse button pressed
			else if( event.button.button == SDL_BUTTON_LEFT && event.button.type == SDL_MOUSEBUTTONDOWN )
			{
				// maybe the first drag click
                prev_mouse_pos->x = mouse->GetX();
                prev_mouse_pos->y = mouse->GetY();
				             
                if(mm_window->IsInside(mouse->GetX(), mouse->GetY()))
					mouse_clicked = true;
			}

			// left mouse button released
			else if( event.button.button == SDL_BUTTON_LEFT && event.button.type == SDL_MOUSEBUTTONUP )
			{
				// click inside the minimap window 
                if(mm_window->IsInside(mouse->GetX(), mouse->GetY()))
				{
                    // and not dragging
                    if(!dragging)
                    {
                        // minimap window is OFF
                        if(!mmap_on)
					    {    					   
                            mmap_on = true;
					        mm_window->SetOpacity(SDL_ALPHA_OPAQUE);
             
                            // update the scene             
                            renderer.RequestBlit(BUFFERED);                            
					    }
					    // minimap window is ON
					    else
					    {        
                            // manage the click on the minimap panel         
                            mm_panel->MouseClicked(mouse->GetX() - mm_window->GetX(), 
                                                   mouse->GetY() - mm_window->GetY());              
                    
                            // update the render position according to minimap panel view
                            renderer.UpdateViewPosition(mm_panel->GetViewX(), mm_panel->GetViewY());
 
                            // update mouse view
                            mouse->SetView(renderer.GetViewX(), renderer.GetViewY());
                    
                            renderer.RequestBufferRefresh();             
                            renderer.RequestBlit(BUFFERED);
                        }
                    }
                }
				// click outside the minimap
				else 
				{    
                    // deactivate the minimap if it's ON
                    if(mmap_on)
                    {
	   				   // turn the minimap window OFF
					   mmap_on = false;
					   mm_window->SetOpacity(TRANS);

                       // update the scene             
                       renderer.RequestBlit(BUFFERED);  
                    }
     
                    //  object selected by the click
                    temp_elem = (Element *)om.SelectObject(mouse->GetAbsX(), mouse->GetAbsY());
                    // cell selected by the click
                    temp_cell = tm.GetCell(mouse->GetAbsX(), mouse->GetAbsY());
                
                    // select a cell outside the map and no object ==> deselect object
                    if(((temp_cell == NULL && temp_elem == NULL) || (sel_elem == temp_elem))
                         && elem_selected && !unit_moving)
                    {            
                        elem_selected = false;
                        
                        sel_elem->Deselect();
                        
                        sel_elem = NULL;
                        sel_unit = NULL;
                        sel_build = NULL;
                            
                        // clear selection
                        sel.ClearSelected();
                        // clear selection
                        sel_data.ClearSelected();
                        
                        // update scene
                        sel.SetBlit(false);
                        sel_data.SetBlit(false);
                        renderer.RequestBlit(BUFFERED);  
                        
                        if(gen_path != NULL)
                        {
                            delete gen_path;
                            gen_path = NULL; 
                            path_h.SetBlit(false);
                        }
                    }                       
                    // select element
                    else if(temp_elem != NULL && temp_elem->GetType() != SCENE_ELEMENT && !unit_moving)
                    {
                        elem_selected = true;
                        
                        if(sel_elem != NULL)
                            sel_elem->Deselect();
                        
                        // store the pointer
                        sel_elem = dynamic_cast <FactionElement *>(temp_elem);
                        
                        if(temp_elem->GetType() == UNIT)
                        {
                            sel_unit = dynamic_cast <Unit *>(temp_elem);
                            sel_build = NULL;
                        }
                        else 
                        {
                            sel_build = dynamic_cast <Building *>(temp_elem);
                            sel_unit = NULL;
                        }
                                                                    
                        // set the selected flag of the objcet
                        sel_elem->Select();
                        
                        // set the selected object in the selection layer
                        sel.SetSelected(sel_elem);
                        sel_data.SetSelected(sel_elem);

                        // update scene
                        sel.SetBlit();
                        sel_data.SetBlit();
                        renderer.RequestBlit(BUFFERED);
                        
                        if(gen_path != NULL)
                        {
                            delete gen_path;
                            gen_path = NULL; 
                            path_h.SetBlit(false);
                        }
                     } 
                                        
                     // start moving
                     else if(gen_path!= NULL && !unit_moving && 
                             sel_unit->GetCurStat(STAT_ACTION) >= gen_path->GetTotalCost() &&
                             sel_unit->GetFaction() == F_NGG)
                     {
                         unit_moving = true;

                         if(sel_unit->GetDirection() != gen_path->GetNode(1)->GetDirection())
                         {
                             sel_unit->SetDirection(gen_path->GetNode(1)->GetDirection());
                                                                        
                            // reset map visibility
                            tm.SetVisibility();
                            // building 
                            om.ComputeObjVisibility(&build1, build1.GetCurStat(STAT_VISIBILITY));
                            // unit 
                            om.ComputeObjVisibility(&unit1, unit1.GetCurStat(STAT_VISIBILITY), unit1.GetDirection());
                            // unit 
                            om.ComputeObjVisibility(&unit2, unit2.GetCurStat(STAT_VISIBILITY), unit2.GetDirection());
                            // unit visibility
                            om.ComputeObjVisibility(&unit3, unit3.GetCurStat(STAT_VISIBILITY), unit3.GetDirection());
                         }
                    
                         delete gen_path->PopNode();

                         renderer.RequestBlit(BUFFERED);
                     }
				}
                                
				// mouse released
				mouse_clicked = false;
				// no dragging
				dragging = false;
            }	
            // right button UP
            else if( event.button.button == SDL_BUTTON_RIGHT && event.button.type == SDL_MOUSEBUTTONUP )
            {
                // rotate unit                                
                if(sel_unit != NULL && !unit_moving  && sel_unit->GetFaction() == F_NGG) 
                {                
                    // relative origin => center of the unit image                 
                    f_origin = iso2scr(sel_unit->GetX() + (sel_unit->GetImage()->GetW() / 2),
                                       sel_unit->GetY() + (sel_unit->GetImage()->GetH() / 2),
                                       tm.GetOriginX(),  tm.GetOriginY());
                    
                    // get orientation
                    orientation = RelDirection((int)round(f_origin->x), (int)round(f_origin->y),
                                               (int)round(mouse->GetAbsOrthoX()), 
                                               (int)round(mouse->GetAbsOrthoY()));
                    
                    delete f_origin;
 
                    // rotate the unit
                    if(sel_unit->GetType() == UNIT &&  orientation != SELF  && 
                       orientation != sel_unit->GetDirection() &&
                       sel_unit->GetCurStat(STAT_ACTION) >= ROTATION_COST)
                    {
                        sel_unit->SetDirection(orientation);
                        
                        sel_unit->SubCurStat(STAT_ACTION, ROTATION_COST);

                        // reset map visibility
                        tm.SetVisibility();
                        // building 
                        om.ComputeObjVisibility(&build1, build1.GetCurStat(STAT_VISIBILITY));
                        // unit 
                        om.ComputeObjVisibility(&unit1, unit1.GetCurStat(STAT_VISIBILITY), unit1.GetDirection());
                        // unit 
                        om.ComputeObjVisibility(&unit2, unit2.GetCurStat(STAT_VISIBILITY), unit2.GetDirection());
                        // unit visibility
                        om.ComputeObjVisibility(&unit3, unit3.GetCurStat(STAT_VISIBILITY), unit3.GetDirection());
                        
                        // update the minimap                        
                        mm.UpdateMap();
                        mm_panel->Refresh();
                                              
                        // update the path
                        if(mouse->IsInsideMap())                             
                        {
                            if(tm.IsWalkable(mouse->GetRow(), mouse->GetCol()) || 
                               tm.IsVisible(mouse->GetRow(), mouse->GetCol()) != VISIBLE)
                            {
                                if(gen_path != NULL)
                                    delete gen_path;
                                                
                                gen_path = path_finder.MakePath(sel_unit->GetRow(), sel_unit->GetCol(), 
                                        mouse->GetRow(), mouse->GetCol(), 
                                        sel_unit->GetDirection());
                        
                                path_h.SetPath(gen_path);
                                                     
                                path_h.SetRemainingAP(sel_unit->GetCurStat(STAT_ACTION) - gen_path->GetTotalCost());
                            }
                        }
                                               
                        renderer.RequestBufferRefresh();
                        renderer.RequestBlit(BUFFERED);                      
                    }
                }
            }
            
		}

		// get times to compute the current pixels movement
		t2_mov = SDL_GetTicks();
		t_diff = t2_mov - t1_mov;
		t1_mov = t2_mov;

        // scrolling
        if(renderer.IsScrolling())
        {            
            renderer.Scrolling(t_diff);
            
            // update mouse view
            mouse->SetView(renderer.GetViewX(), renderer.GetViewY());
            // update view
            mm_panel->UpdateView(renderer.GetViewX(), renderer.GetViewY());
        }
        
        // a unit is moving
        if(unit_moving)            
        {    
            // not blit selection highlighter
            sel.SetBlit(false);
            sel_data.SetBlit(false);
            
            // it needs to get the next path node
            if(cur_pnode == NULL)
            {
                cur_pnode = gen_path->PopNode();  

                if(sel_unit->GetDirection() != cur_pnode->GetDirection())                  
                    sel_unit->SetDirection(cur_pnode->GetDirection());
                
                if(gen_path->GetPathLen() == 0)
                {
                    gen_path = NULL;
                    path_h.SetBlit(false);
                }

                // the current node is not walkable
                if(!tm.IsWalkable(cur_pnode->GetRow(), cur_pnode->GetCol()))
                {
                    delete cur_pnode;
                    cur_pnode = NULL;
                    
                    // stop moving
                    unit_moving = false;           
                    
                    delete gen_path;         
                    gen_path = NULL;
                    path_h.SetBlit(false);
                    
                    checkp_reached = true;
                    
                    // reset map visibility
                    tm.SetVisibility();
                    // building 
                    om.ComputeObjVisibility(&build1, build1.GetCurStat(STAT_VISIBILITY));
                    // unit 
                    om.ComputeObjVisibility(&unit1, unit1.GetCurStat(STAT_VISIBILITY), unit1.GetDirection());
                    // unit 
                    om.ComputeObjVisibility(&unit2, unit2.GetCurStat(STAT_VISIBILITY), unit2.GetDirection());
                    // unit visibility
                    om.ComputeObjVisibility(&unit3, unit3.GetCurStat(STAT_VISIBILITY), unit3.GetDirection());
                    
                    // update the minimap                    
                    mm.UpdateMap();
                    
                    mm_panel->Refresh();

                    renderer.RequestBufferRefresh();
                    renderer.RequestBlit(BUFFERED);
                }
                // move to the node
                else
                {                    
                    sel_unit->SubCurStat(STAT_ACTION, cur_pnode->GetCost());
                    
                    checkp_reached = false;

                    om.MoveObject(sel_unit, cur_pnode->GetRow(), cur_pnode->GetCol());

                    x_dest = om.GetObjX(sel_unit);
                    y_dest = om.GetObjY(sel_unit);

                    // reset map visibility
                    tm.SetVisibility();
                    // building 
                    om.ComputeObjVisibility(&build1, build1.GetCurStat(STAT_VISIBILITY));
                    // unit 
                    om.ComputeObjVisibility(&unit1, unit1.GetCurStat(STAT_VISIBILITY), unit1.GetDirection());
                    // unit 
                    om.ComputeObjVisibility(&unit2, unit2.GetCurStat(STAT_VISIBILITY), unit2.GetDirection());
                    // unit visibility
                    om.ComputeObjVisibility(&unit3, unit3.GetCurStat(STAT_VISIBILITY), unit3.GetDirection());
                        
                    // update the minimap                    
                    mm.UpdateMap();                    
                    mm_panel->Refresh();
 
                    renderer.RequestBufferRefresh();
                    renderer.RequestBlit(BUFFERED);
                }
            }
                                     
            // moving toward the next cell
            if(!checkp_reached)                
            {                   
                checkp_reached = sel_unit->MoveTo(x_dest, y_dest, OBJ_XMOV, OBJ_YMOV);
                renderer.RequestBlit(BUFFERED);
            }
            // cell reached
            else
            {                                          
                delete cur_pnode;
                cur_pnode = NULL;
                
                checkp_reached = false;
                                
                // it was the last cell of the path
                if(gen_path == NULL)
                {
                    unit_moving = false;      
                    path_h.SetBlit(false);
                    
                    // reblit the selection highlighter
                    sel.SetBlit(true);
                    sel_data.SetBlit(true);
                    
                    renderer.RequestBlit(BUFFERED);
                }
            }                        
        }
        
        if(renderer.NeedBufferRefresh())
            renderer.RefreshBuffer();            
                      
        // blit of the scene
        if(renderer.NeedBlit())
        {            
           renderer.BlitScene(*screen);
           
           // MINIMAP         
           // save background (screen) under the minimap window
           mm_border->SaveBackground(screen);              
           mm_panel->Blit(mm_window);
           // blit window border
           blitter->Blit(*mm_border);   
           // blit window content
           blitter->Blit(*mm_window); 
                          
           // mouse pointer management
           mouse->Blit();
           
           screen->Update();
        }
        // buffered blit of the scene
        else if(renderer.NeedBufferedBlit())
        {            
            renderer.BufferedBlitScene(*screen);
            
            // MINIMAP         
            // save background (screen) under the minimap window
            mm_border->SaveBackground(screen);                
            mm_panel->Blit(mm_window);
            // blit window border
            blitter->Blit(*mm_border);   
            // blit window content
            blitter->Blit(*mm_window); 
            
            // mouse pointer management
            mouse->Blit();

            screen->Update();
        }
        	
		// compute delay time
		t2 = SDL_GetTicks();
		t_delay = FRAME_TIME - (t2 - t1);
		t1 = t2;
        if(t_delay > 0 )
			SDL_Delay(t_delay);
	}
	
	return 0;
}
